'use strict';

var web3_js = require('@solana/web3.js');
var jsSha3 = require('js-sha3');
var Collections = require('typescript-collections');
var mplTokenMetadata = require('@tensor-foundation/mpl-token-metadata');
var testHelpers = require('@tensor-foundation/test-helpers');
var system = require('@solana-program/system');
var computeBudget = require('@solana-program/compute-budget');

function _interopNamespace(e) {
  if (e && e.__esModule) return e;
  var n = Object.create(null);
  if (e) {
    Object.keys(e).forEach(function (k) {
      if (k !== 'default') {
        var d = Object.getOwnPropertyDescriptor(e, k);
        Object.defineProperty(n, k, d.get ? d : {
          enumerable: true,
          get: function () { return e[k]; }
        });
      }
    });
  }
  n.default = e;
  return Object.freeze(n);
}

var Collections__namespace = /*#__PURE__*/_interopNamespace(Collections);

// env-shim.ts
var __DEV__ = /* @__PURE__ */ (() => process["env"].NODE_ENV === "development")();

// src/account_compression/generated/errors/splAccountCompression.ts
var SPL_ACCOUNT_COMPRESSION_ERROR__INCORRECT_LEAF_LENGTH = 6e3;
var SPL_ACCOUNT_COMPRESSION_ERROR__CONCURRENT_MERKLE_TREE_ERROR = 6001;
var SPL_ACCOUNT_COMPRESSION_ERROR__ZERO_COPY_ERROR = 6002;
var SPL_ACCOUNT_COMPRESSION_ERROR__CONCURRENT_MERKLE_TREE_CONSTANTS_ERROR = 6003;
var SPL_ACCOUNT_COMPRESSION_ERROR__CANOPY_LENGTH_MISMATCH = 6004;
var SPL_ACCOUNT_COMPRESSION_ERROR__INCORRECT_AUTHORITY = 6005;
var SPL_ACCOUNT_COMPRESSION_ERROR__INCORRECT_ACCOUNT_OWNER = 6006;
var SPL_ACCOUNT_COMPRESSION_ERROR__INCORRECT_ACCOUNT_TYPE = 6007;
var SPL_ACCOUNT_COMPRESSION_ERROR__LEAF_INDEX_OUT_OF_BOUNDS = 6008;
var splAccountCompressionErrorMessages;
if (__DEV__) {
  splAccountCompressionErrorMessages = {
    [SPL_ACCOUNT_COMPRESSION_ERROR__CANOPY_LENGTH_MISMATCH]: `Expected a different byte length for the merkle tree canopy`,
    [SPL_ACCOUNT_COMPRESSION_ERROR__CONCURRENT_MERKLE_TREE_CONSTANTS_ERROR]: `An unsupported max depth or max buffer size constant was provided`,
    [SPL_ACCOUNT_COMPRESSION_ERROR__CONCURRENT_MERKLE_TREE_ERROR]: `Concurrent merkle tree error`,
    [SPL_ACCOUNT_COMPRESSION_ERROR__INCORRECT_ACCOUNT_OWNER]: `Account is owned by a different program, expected it to be owned by this program`,
    [SPL_ACCOUNT_COMPRESSION_ERROR__INCORRECT_ACCOUNT_TYPE]: `Account provided has incorrect account type`,
    [SPL_ACCOUNT_COMPRESSION_ERROR__INCORRECT_AUTHORITY]: `Provided authority does not match expected tree authority`,
    [SPL_ACCOUNT_COMPRESSION_ERROR__INCORRECT_LEAF_LENGTH]: `Incorrect leaf length. Expected vec of 32 bytes`,
    [SPL_ACCOUNT_COMPRESSION_ERROR__LEAF_INDEX_OUT_OF_BOUNDS]: `Leaf index of concurrent merkle tree is out of bounds`,
    [SPL_ACCOUNT_COMPRESSION_ERROR__ZERO_COPY_ERROR]: `Issue zero copying concurrent merkle tree data`
  };
}
function getSplAccountCompressionErrorMessage(code) {
  if (__DEV__) {
    return splAccountCompressionErrorMessages[code];
  }
  return "Error message not available in production bundles. Compile with `__DEV__` set to true to see more information.";
}
var SPL_ACCOUNT_COMPRESSION_PROGRAM_ADDRESS = "cmtDvXumGCrqC1Age74AVPhSRVXJMd8PJS91L8KbNCK";
var SplAccountCompressionInstruction = /* @__PURE__ */ ((SplAccountCompressionInstruction2) => {
  SplAccountCompressionInstruction2[SplAccountCompressionInstruction2["InitEmptyMerkleTree"] = 0] = "InitEmptyMerkleTree";
  SplAccountCompressionInstruction2[SplAccountCompressionInstruction2["ReplaceLeaf"] = 1] = "ReplaceLeaf";
  SplAccountCompressionInstruction2[SplAccountCompressionInstruction2["TransferAuthority"] = 2] = "TransferAuthority";
  SplAccountCompressionInstruction2[SplAccountCompressionInstruction2["VerifyLeaf"] = 3] = "VerifyLeaf";
  SplAccountCompressionInstruction2[SplAccountCompressionInstruction2["Append"] = 4] = "Append";
  SplAccountCompressionInstruction2[SplAccountCompressionInstruction2["InsertOrAppend"] = 5] = "InsertOrAppend";
  SplAccountCompressionInstruction2[SplAccountCompressionInstruction2["CloseEmptyTree"] = 6] = "CloseEmptyTree";
  return SplAccountCompressionInstruction2;
})(SplAccountCompressionInstruction || {});
function identifySplAccountCompressionInstruction(instruction) {
  const data = instruction instanceof Uint8Array ? instruction : instruction.data;
  if (web3_js.containsBytes(
    data,
    web3_js.fixEncoderSize(web3_js.getBytesEncoder(), 8).encode(
      new Uint8Array([191, 11, 119, 7, 180, 107, 220, 110])
    ),
    0
  )) {
    return 0 /* InitEmptyMerkleTree */;
  }
  if (web3_js.containsBytes(
    data,
    web3_js.fixEncoderSize(web3_js.getBytesEncoder(), 8).encode(
      new Uint8Array([204, 165, 76, 100, 73, 147, 0, 128])
    ),
    0
  )) {
    return 1 /* ReplaceLeaf */;
  }
  if (web3_js.containsBytes(
    data,
    web3_js.fixEncoderSize(web3_js.getBytesEncoder(), 8).encode(
      new Uint8Array([48, 169, 76, 72, 229, 180, 55, 161])
    ),
    0
  )) {
    return 2 /* TransferAuthority */;
  }
  if (web3_js.containsBytes(
    data,
    web3_js.fixEncoderSize(web3_js.getBytesEncoder(), 8).encode(
      new Uint8Array([124, 220, 22, 223, 104, 10, 250, 224])
    ),
    0
  )) {
    return 3 /* VerifyLeaf */;
  }
  if (web3_js.containsBytes(
    data,
    web3_js.fixEncoderSize(web3_js.getBytesEncoder(), 8).encode(
      new Uint8Array([149, 120, 18, 222, 236, 225, 88, 203])
    ),
    0
  )) {
    return 4 /* Append */;
  }
  if (web3_js.containsBytes(
    data,
    web3_js.fixEncoderSize(web3_js.getBytesEncoder(), 8).encode(
      new Uint8Array([6, 42, 50, 190, 51, 109, 178, 168])
    ),
    0
  )) {
    return 5 /* InsertOrAppend */;
  }
  if (web3_js.containsBytes(
    data,
    web3_js.fixEncoderSize(web3_js.getBytesEncoder(), 8).encode(
      new Uint8Array([50, 14, 219, 107, 78, 103, 16, 103])
    ),
    0
  )) {
    return 6 /* CloseEmptyTree */;
  }
  throw new Error(
    "The provided instruction could not be identified as a splAccountCompression instruction."
  );
}
function expectAddress(value) {
  if (!value) {
    throw new Error("Expected a Address.");
  }
  if (typeof value === "object" && "address" in value) {
    return value.address;
  }
  if (Array.isArray(value)) {
    return value[0];
  }
  return value;
}
function getAccountMetaFactory(programAddress, optionalAccountStrategy) {
  return (account) => {
    if (!account.value) {
      return Object.freeze({
        address: programAddress,
        role: web3_js.AccountRole.READONLY
      });
    }
    const writableRole = account.isWritable ? web3_js.AccountRole.WRITABLE : web3_js.AccountRole.READONLY;
    return Object.freeze({
      address: expectAddress(account.value),
      role: isTransactionSigner(account.value) ? web3_js.upgradeRoleToSigner(writableRole) : writableRole,
      ...isTransactionSigner(account.value) ? { signer: account.value } : {}
    });
  };
}
function isTransactionSigner(value) {
  return !!value && typeof value === "object" && "address" in value && web3_js.isTransactionSigner(value);
}

// src/account_compression/generated/instructions/append.ts
function getAppendInstructionDataEncoder() {
  return web3_js.transformEncoder(
    web3_js.getStructEncoder([
      ["discriminator", web3_js.fixEncoderSize(web3_js.getBytesEncoder(), 8)],
      ["leaf", web3_js.fixEncoderSize(web3_js.getBytesEncoder(), 32)]
    ]),
    (value) => ({
      ...value,
      discriminator: new Uint8Array([149, 120, 18, 222, 236, 225, 88, 203])
    })
  );
}
function getAppendInstructionDataDecoder() {
  return web3_js.getStructDecoder([
    ["discriminator", web3_js.fixDecoderSize(web3_js.getBytesDecoder(), 8)],
    ["leaf", web3_js.fixDecoderSize(web3_js.getBytesDecoder(), 32)]
  ]);
}
function getAppendInstructionDataCodec() {
  return web3_js.combineCodec(
    getAppendInstructionDataEncoder(),
    getAppendInstructionDataDecoder()
  );
}
function getAppendInstruction(input) {
  const programAddress = SPL_ACCOUNT_COMPRESSION_PROGRAM_ADDRESS;
  const originalAccounts = {
    merkleTree: { value: input.merkleTree ?? null, isWritable: true },
    authority: { value: input.authority ?? null, isWritable: false },
    noop: { value: input.noop ?? null, isWritable: false }
  };
  const accounts = originalAccounts;
  const args = { ...input };
  const getAccountMeta = getAccountMetaFactory(programAddress);
  const instruction = {
    accounts: [
      getAccountMeta(accounts.merkleTree),
      getAccountMeta(accounts.authority),
      getAccountMeta(accounts.noop)
    ],
    programAddress,
    data: getAppendInstructionDataEncoder().encode(
      args
    )
  };
  return instruction;
}
function parseAppendInstruction(instruction) {
  if (instruction.accounts.length < 3) {
    throw new Error("Not enough accounts");
  }
  let accountIndex = 0;
  const getNextAccount = () => {
    const accountMeta = instruction.accounts[accountIndex];
    accountIndex += 1;
    return accountMeta;
  };
  return {
    programAddress: instruction.programAddress,
    accounts: {
      merkleTree: getNextAccount(),
      authority: getNextAccount(),
      noop: getNextAccount()
    },
    data: getAppendInstructionDataDecoder().decode(instruction.data)
  };
}
function getCloseEmptyTreeInstructionDataEncoder() {
  return web3_js.transformEncoder(
    web3_js.getStructEncoder([["discriminator", web3_js.fixEncoderSize(web3_js.getBytesEncoder(), 8)]]),
    (value) => ({
      ...value,
      discriminator: new Uint8Array([50, 14, 219, 107, 78, 103, 16, 103])
    })
  );
}
function getCloseEmptyTreeInstructionDataDecoder() {
  return web3_js.getStructDecoder([
    ["discriminator", web3_js.fixDecoderSize(web3_js.getBytesDecoder(), 8)]
  ]);
}
function getCloseEmptyTreeInstructionDataCodec() {
  return web3_js.combineCodec(
    getCloseEmptyTreeInstructionDataEncoder(),
    getCloseEmptyTreeInstructionDataDecoder()
  );
}
function getCloseEmptyTreeInstruction(input) {
  const programAddress = SPL_ACCOUNT_COMPRESSION_PROGRAM_ADDRESS;
  const originalAccounts = {
    merkleTree: { value: input.merkleTree ?? null, isWritable: true },
    authority: { value: input.authority ?? null, isWritable: false },
    recipient: { value: input.recipient ?? null, isWritable: true }
  };
  const accounts = originalAccounts;
  const getAccountMeta = getAccountMetaFactory(programAddress);
  const instruction = {
    accounts: [
      getAccountMeta(accounts.merkleTree),
      getAccountMeta(accounts.authority),
      getAccountMeta(accounts.recipient)
    ],
    programAddress,
    data: getCloseEmptyTreeInstructionDataEncoder().encode({})
  };
  return instruction;
}
function parseCloseEmptyTreeInstruction(instruction) {
  if (instruction.accounts.length < 3) {
    throw new Error("Not enough accounts");
  }
  let accountIndex = 0;
  const getNextAccount = () => {
    const accountMeta = instruction.accounts[accountIndex];
    accountIndex += 1;
    return accountMeta;
  };
  return {
    programAddress: instruction.programAddress,
    accounts: {
      merkleTree: getNextAccount(),
      authority: getNextAccount(),
      recipient: getNextAccount()
    },
    data: getCloseEmptyTreeInstructionDataDecoder().decode(instruction.data)
  };
}
function getInitEmptyMerkleTreeInstructionDataEncoder() {
  return web3_js.transformEncoder(
    web3_js.getStructEncoder([
      ["discriminator", web3_js.fixEncoderSize(web3_js.getBytesEncoder(), 8)],
      ["maxDepth", web3_js.getU32Encoder()],
      ["maxBufferSize", web3_js.getU32Encoder()]
    ]),
    (value) => ({
      ...value,
      discriminator: new Uint8Array([191, 11, 119, 7, 180, 107, 220, 110])
    })
  );
}
function getInitEmptyMerkleTreeInstructionDataDecoder() {
  return web3_js.getStructDecoder([
    ["discriminator", web3_js.fixDecoderSize(web3_js.getBytesDecoder(), 8)],
    ["maxDepth", web3_js.getU32Decoder()],
    ["maxBufferSize", web3_js.getU32Decoder()]
  ]);
}
function getInitEmptyMerkleTreeInstructionDataCodec() {
  return web3_js.combineCodec(
    getInitEmptyMerkleTreeInstructionDataEncoder(),
    getInitEmptyMerkleTreeInstructionDataDecoder()
  );
}
function getInitEmptyMerkleTreeInstruction(input) {
  const programAddress = SPL_ACCOUNT_COMPRESSION_PROGRAM_ADDRESS;
  const originalAccounts = {
    merkleTree: { value: input.merkleTree ?? null, isWritable: true },
    authority: { value: input.authority ?? null, isWritable: false },
    noop: { value: input.noop ?? null, isWritable: false }
  };
  const accounts = originalAccounts;
  const args = { ...input };
  const getAccountMeta = getAccountMetaFactory(programAddress);
  const instruction = {
    accounts: [
      getAccountMeta(accounts.merkleTree),
      getAccountMeta(accounts.authority),
      getAccountMeta(accounts.noop)
    ],
    programAddress,
    data: getInitEmptyMerkleTreeInstructionDataEncoder().encode(
      args
    )
  };
  return instruction;
}
function parseInitEmptyMerkleTreeInstruction(instruction) {
  if (instruction.accounts.length < 3) {
    throw new Error("Not enough accounts");
  }
  let accountIndex = 0;
  const getNextAccount = () => {
    const accountMeta = instruction.accounts[accountIndex];
    accountIndex += 1;
    return accountMeta;
  };
  return {
    programAddress: instruction.programAddress,
    accounts: {
      merkleTree: getNextAccount(),
      authority: getNextAccount(),
      noop: getNextAccount()
    },
    data: getInitEmptyMerkleTreeInstructionDataDecoder().decode(
      instruction.data
    )
  };
}
function getInsertOrAppendInstructionDataEncoder() {
  return web3_js.transformEncoder(
    web3_js.getStructEncoder([
      ["discriminator", web3_js.fixEncoderSize(web3_js.getBytesEncoder(), 8)],
      ["root", web3_js.fixEncoderSize(web3_js.getBytesEncoder(), 32)],
      ["leaf", web3_js.fixEncoderSize(web3_js.getBytesEncoder(), 32)],
      ["index", web3_js.getU32Encoder()]
    ]),
    (value) => ({
      ...value,
      discriminator: new Uint8Array([6, 42, 50, 190, 51, 109, 178, 168])
    })
  );
}
function getInsertOrAppendInstructionDataDecoder() {
  return web3_js.getStructDecoder([
    ["discriminator", web3_js.fixDecoderSize(web3_js.getBytesDecoder(), 8)],
    ["root", web3_js.fixDecoderSize(web3_js.getBytesDecoder(), 32)],
    ["leaf", web3_js.fixDecoderSize(web3_js.getBytesDecoder(), 32)],
    ["index", web3_js.getU32Decoder()]
  ]);
}
function getInsertOrAppendInstructionDataCodec() {
  return web3_js.combineCodec(
    getInsertOrAppendInstructionDataEncoder(),
    getInsertOrAppendInstructionDataDecoder()
  );
}
function getInsertOrAppendInstruction(input) {
  const programAddress = SPL_ACCOUNT_COMPRESSION_PROGRAM_ADDRESS;
  const originalAccounts = {
    merkleTree: { value: input.merkleTree ?? null, isWritable: true },
    authority: { value: input.authority ?? null, isWritable: false },
    noop: { value: input.noop ?? null, isWritable: false }
  };
  const accounts = originalAccounts;
  const args = { ...input };
  const getAccountMeta = getAccountMetaFactory(programAddress);
  const instruction = {
    accounts: [
      getAccountMeta(accounts.merkleTree),
      getAccountMeta(accounts.authority),
      getAccountMeta(accounts.noop)
    ],
    programAddress,
    data: getInsertOrAppendInstructionDataEncoder().encode(
      args
    )
  };
  return instruction;
}
function parseInsertOrAppendInstruction(instruction) {
  if (instruction.accounts.length < 3) {
    throw new Error("Not enough accounts");
  }
  let accountIndex = 0;
  const getNextAccount = () => {
    const accountMeta = instruction.accounts[accountIndex];
    accountIndex += 1;
    return accountMeta;
  };
  return {
    programAddress: instruction.programAddress,
    accounts: {
      merkleTree: getNextAccount(),
      authority: getNextAccount(),
      noop: getNextAccount()
    },
    data: getInsertOrAppendInstructionDataDecoder().decode(instruction.data)
  };
}
function getReplaceLeafInstructionDataEncoder() {
  return web3_js.transformEncoder(
    web3_js.getStructEncoder([
      ["discriminator", web3_js.fixEncoderSize(web3_js.getBytesEncoder(), 8)],
      ["root", web3_js.fixEncoderSize(web3_js.getBytesEncoder(), 32)],
      ["previousLeaf", web3_js.fixEncoderSize(web3_js.getBytesEncoder(), 32)],
      ["newLeaf", web3_js.fixEncoderSize(web3_js.getBytesEncoder(), 32)],
      ["index", web3_js.getU32Encoder()]
    ]),
    (value) => ({
      ...value,
      discriminator: new Uint8Array([204, 165, 76, 100, 73, 147, 0, 128])
    })
  );
}
function getReplaceLeafInstructionDataDecoder() {
  return web3_js.getStructDecoder([
    ["discriminator", web3_js.fixDecoderSize(web3_js.getBytesDecoder(), 8)],
    ["root", web3_js.fixDecoderSize(web3_js.getBytesDecoder(), 32)],
    ["previousLeaf", web3_js.fixDecoderSize(web3_js.getBytesDecoder(), 32)],
    ["newLeaf", web3_js.fixDecoderSize(web3_js.getBytesDecoder(), 32)],
    ["index", web3_js.getU32Decoder()]
  ]);
}
function getReplaceLeafInstructionDataCodec() {
  return web3_js.combineCodec(
    getReplaceLeafInstructionDataEncoder(),
    getReplaceLeafInstructionDataDecoder()
  );
}
function getReplaceLeafInstruction(input) {
  const programAddress = SPL_ACCOUNT_COMPRESSION_PROGRAM_ADDRESS;
  const originalAccounts = {
    merkleTree: { value: input.merkleTree ?? null, isWritable: true },
    authority: { value: input.authority ?? null, isWritable: false },
    noop: { value: input.noop ?? null, isWritable: false }
  };
  const accounts = originalAccounts;
  const args = { ...input };
  const getAccountMeta = getAccountMetaFactory(programAddress);
  const instruction = {
    accounts: [
      getAccountMeta(accounts.merkleTree),
      getAccountMeta(accounts.authority),
      getAccountMeta(accounts.noop)
    ],
    programAddress,
    data: getReplaceLeafInstructionDataEncoder().encode(
      args
    )
  };
  return instruction;
}
function parseReplaceLeafInstruction(instruction) {
  if (instruction.accounts.length < 3) {
    throw new Error("Not enough accounts");
  }
  let accountIndex = 0;
  const getNextAccount = () => {
    const accountMeta = instruction.accounts[accountIndex];
    accountIndex += 1;
    return accountMeta;
  };
  return {
    programAddress: instruction.programAddress,
    accounts: {
      merkleTree: getNextAccount(),
      authority: getNextAccount(),
      noop: getNextAccount()
    },
    data: getReplaceLeafInstructionDataDecoder().decode(instruction.data)
  };
}
function getTransferAuthorityInstructionDataEncoder() {
  return web3_js.transformEncoder(
    web3_js.getStructEncoder([
      ["discriminator", web3_js.fixEncoderSize(web3_js.getBytesEncoder(), 8)],
      ["newAuthority", web3_js.getAddressEncoder()]
    ]),
    (value) => ({
      ...value,
      discriminator: new Uint8Array([48, 169, 76, 72, 229, 180, 55, 161])
    })
  );
}
function getTransferAuthorityInstructionDataDecoder() {
  return web3_js.getStructDecoder([
    ["discriminator", web3_js.fixDecoderSize(web3_js.getBytesDecoder(), 8)],
    ["newAuthority", web3_js.getAddressDecoder()]
  ]);
}
function getTransferAuthorityInstructionDataCodec() {
  return web3_js.combineCodec(
    getTransferAuthorityInstructionDataEncoder(),
    getTransferAuthorityInstructionDataDecoder()
  );
}
function getTransferAuthorityInstruction(input) {
  const programAddress = SPL_ACCOUNT_COMPRESSION_PROGRAM_ADDRESS;
  const originalAccounts = {
    merkleTree: { value: input.merkleTree ?? null, isWritable: true },
    authority: { value: input.authority ?? null, isWritable: false }
  };
  const accounts = originalAccounts;
  const args = { ...input };
  const getAccountMeta = getAccountMetaFactory(programAddress);
  const instruction = {
    accounts: [
      getAccountMeta(accounts.merkleTree),
      getAccountMeta(accounts.authority)
    ],
    programAddress,
    data: getTransferAuthorityInstructionDataEncoder().encode(
      args
    )
  };
  return instruction;
}
function parseTransferAuthorityInstruction(instruction) {
  if (instruction.accounts.length < 2) {
    throw new Error("Not enough accounts");
  }
  let accountIndex = 0;
  const getNextAccount = () => {
    const accountMeta = instruction.accounts[accountIndex];
    accountIndex += 1;
    return accountMeta;
  };
  return {
    programAddress: instruction.programAddress,
    accounts: {
      merkleTree: getNextAccount(),
      authority: getNextAccount()
    },
    data: getTransferAuthorityInstructionDataDecoder().decode(instruction.data)
  };
}
function getVerifyLeafInstructionDataEncoder() {
  return web3_js.transformEncoder(
    web3_js.getStructEncoder([
      ["discriminator", web3_js.fixEncoderSize(web3_js.getBytesEncoder(), 8)],
      ["root", web3_js.fixEncoderSize(web3_js.getBytesEncoder(), 32)],
      ["leaf", web3_js.fixEncoderSize(web3_js.getBytesEncoder(), 32)],
      ["index", web3_js.getU32Encoder()]
    ]),
    (value) => ({
      ...value,
      discriminator: new Uint8Array([124, 220, 22, 223, 104, 10, 250, 224])
    })
  );
}
function getVerifyLeafInstructionDataDecoder() {
  return web3_js.getStructDecoder([
    ["discriminator", web3_js.fixDecoderSize(web3_js.getBytesDecoder(), 8)],
    ["root", web3_js.fixDecoderSize(web3_js.getBytesDecoder(), 32)],
    ["leaf", web3_js.fixDecoderSize(web3_js.getBytesDecoder(), 32)],
    ["index", web3_js.getU32Decoder()]
  ]);
}
function getVerifyLeafInstructionDataCodec() {
  return web3_js.combineCodec(
    getVerifyLeafInstructionDataEncoder(),
    getVerifyLeafInstructionDataDecoder()
  );
}
function getVerifyLeafInstruction(input) {
  const programAddress = SPL_ACCOUNT_COMPRESSION_PROGRAM_ADDRESS;
  const originalAccounts = {
    merkleTree: { value: input.merkleTree ?? null, isWritable: false }
  };
  const accounts = originalAccounts;
  const args = { ...input };
  const getAccountMeta = getAccountMetaFactory(programAddress);
  const instruction = {
    accounts: [getAccountMeta(accounts.merkleTree)],
    programAddress,
    data: getVerifyLeafInstructionDataEncoder().encode(
      args
    )
  };
  return instruction;
}
function parseVerifyLeafInstruction(instruction) {
  if (instruction.accounts.length < 1) {
    throw new Error("Not enough accounts");
  }
  let accountIndex = 0;
  const getNextAccount = () => {
    const accountMeta = instruction.accounts[accountIndex];
    accountIndex += 1;
    return accountMeta;
  };
  return {
    programAddress: instruction.programAddress,
    accounts: {
      merkleTree: getNextAccount()
    },
    data: getVerifyLeafInstructionDataDecoder().decode(instruction.data)
  };
}
function getAccountCompressionEventEncoder() {
  return web3_js.getDiscriminatedUnionEncoder([
    [
      "ChangeLog",
      web3_js.getStructEncoder([
        ["fields", web3_js.getTupleEncoder([getChangeLogEventEncoder()])]
      ])
    ],
    [
      "ApplicationData",
      web3_js.getStructEncoder([
        ["fields", web3_js.getTupleEncoder([getApplicationDataEventEncoder()])]
      ])
    ]
  ]);
}
function getAccountCompressionEventDecoder() {
  return web3_js.getDiscriminatedUnionDecoder([
    [
      "ChangeLog",
      web3_js.getStructDecoder([
        ["fields", web3_js.getTupleDecoder([getChangeLogEventDecoder()])]
      ])
    ],
    [
      "ApplicationData",
      web3_js.getStructDecoder([
        ["fields", web3_js.getTupleDecoder([getApplicationDataEventDecoder()])]
      ])
    ]
  ]);
}
function getAccountCompressionEventCodec() {
  return web3_js.combineCodec(
    getAccountCompressionEventEncoder(),
    getAccountCompressionEventDecoder()
  );
}
function accountCompressionEvent(kind, data) {
  return Array.isArray(data) ? { __kind: kind, fields: data } : { __kind: kind, ...data ?? {} };
}
function isAccountCompressionEvent(kind, value) {
  return value.__kind === kind;
}
function getApplicationDataEventEncoder() {
  return web3_js.getDiscriminatedUnionEncoder([
    [
      "V1",
      web3_js.getStructEncoder([
        ["fields", web3_js.getTupleEncoder([getApplicationDataEventV1Encoder()])]
      ])
    ]
  ]);
}
function getApplicationDataEventDecoder() {
  return web3_js.getDiscriminatedUnionDecoder([
    [
      "V1",
      web3_js.getStructDecoder([
        ["fields", web3_js.getTupleDecoder([getApplicationDataEventV1Decoder()])]
      ])
    ]
  ]);
}
function getApplicationDataEventCodec() {
  return web3_js.combineCodec(
    getApplicationDataEventEncoder(),
    getApplicationDataEventDecoder()
  );
}
function applicationDataEvent(kind, data) {
  return Array.isArray(data) ? { __kind: kind, fields: data } : { __kind: kind, ...data ?? {} };
}
function isApplicationDataEvent(kind, value) {
  return value.__kind === kind;
}
function getApplicationDataEventV1Encoder() {
  return web3_js.getStructEncoder([
    [
      "applicationData",
      web3_js.addEncoderSizePrefix(web3_js.getBytesEncoder(), web3_js.getU32Encoder())
    ]
  ]);
}
function getApplicationDataEventV1Decoder() {
  return web3_js.getStructDecoder([
    [
      "applicationData",
      web3_js.addDecoderSizePrefix(web3_js.getBytesDecoder(), web3_js.getU32Decoder())
    ]
  ]);
}
function getApplicationDataEventV1Codec() {
  return web3_js.combineCodec(
    getApplicationDataEventV1Encoder(),
    getApplicationDataEventV1Decoder()
  );
}
function getChangeLogEventEncoder() {
  return web3_js.getDiscriminatedUnionEncoder([
    [
      "V1",
      web3_js.getStructEncoder([
        ["fields", web3_js.getTupleEncoder([getChangeLogEventV1Encoder()])]
      ])
    ]
  ]);
}
function getChangeLogEventDecoder() {
  return web3_js.getDiscriminatedUnionDecoder([
    [
      "V1",
      web3_js.getStructDecoder([
        ["fields", web3_js.getTupleDecoder([getChangeLogEventV1Decoder()])]
      ])
    ]
  ]);
}
function getChangeLogEventCodec() {
  return web3_js.combineCodec(getChangeLogEventEncoder(), getChangeLogEventDecoder());
}
function changeLogEvent(kind, data) {
  return Array.isArray(data) ? { __kind: kind, fields: data } : { __kind: kind, ...data ?? {} };
}
function isChangeLogEvent(kind, value) {
  return value.__kind === kind;
}
function getChangeLogEventV1Encoder() {
  return web3_js.getStructEncoder([
    ["id", web3_js.getAddressEncoder()],
    ["path", web3_js.getArrayEncoder(getPathNodeEncoder())],
    ["seq", web3_js.getU64Encoder()],
    ["index", web3_js.getU32Encoder()]
  ]);
}
function getChangeLogEventV1Decoder() {
  return web3_js.getStructDecoder([
    ["id", web3_js.getAddressDecoder()],
    ["path", web3_js.getArrayDecoder(getPathNodeDecoder())],
    ["seq", web3_js.getU64Decoder()],
    ["index", web3_js.getU32Decoder()]
  ]);
}
function getChangeLogEventV1Codec() {
  return web3_js.combineCodec(
    getChangeLogEventV1Encoder(),
    getChangeLogEventV1Decoder()
  );
}
var CompressionAccountType = /* @__PURE__ */ ((CompressionAccountType3) => {
  CompressionAccountType3[CompressionAccountType3["Uninitialized"] = 0] = "Uninitialized";
  CompressionAccountType3[CompressionAccountType3["ConcurrentMerkleTree"] = 1] = "ConcurrentMerkleTree";
  return CompressionAccountType3;
})(CompressionAccountType || {});
function getCompressionAccountTypeEncoder() {
  return web3_js.getEnumEncoder(CompressionAccountType);
}
function getCompressionAccountTypeDecoder() {
  return web3_js.getEnumDecoder(CompressionAccountType);
}
function getCompressionAccountTypeCodec() {
  return web3_js.combineCodec(
    getCompressionAccountTypeEncoder(),
    getCompressionAccountTypeDecoder()
  );
}
function getConcurrentMerkleTreeHeaderEncoder() {
  return web3_js.getStructEncoder([
    ["accountType", getCompressionAccountTypeEncoder()],
    ["header", getConcurrentMerkleTreeHeaderDataEncoder()]
  ]);
}
function getConcurrentMerkleTreeHeaderDecoder() {
  return web3_js.getStructDecoder([
    ["accountType", getCompressionAccountTypeDecoder()],
    ["header", getConcurrentMerkleTreeHeaderDataDecoder()]
  ]);
}
function getConcurrentMerkleTreeHeaderCodec() {
  return web3_js.combineCodec(
    getConcurrentMerkleTreeHeaderEncoder(),
    getConcurrentMerkleTreeHeaderDecoder()
  );
}
function getConcurrentMerkleTreeHeaderDataEncoder() {
  return web3_js.getDiscriminatedUnionEncoder([
    [
      "V1",
      web3_js.getStructEncoder([
        [
          "fields",
          web3_js.getTupleEncoder([getConcurrentMerkleTreeHeaderDataV1Encoder()])
        ]
      ])
    ]
  ]);
}
function getConcurrentMerkleTreeHeaderDataDecoder() {
  return web3_js.getDiscriminatedUnionDecoder([
    [
      "V1",
      web3_js.getStructDecoder([
        [
          "fields",
          web3_js.getTupleDecoder([getConcurrentMerkleTreeHeaderDataV1Decoder()])
        ]
      ])
    ]
  ]);
}
function getConcurrentMerkleTreeHeaderDataCodec() {
  return web3_js.combineCodec(
    getConcurrentMerkleTreeHeaderDataEncoder(),
    getConcurrentMerkleTreeHeaderDataDecoder()
  );
}
function concurrentMerkleTreeHeaderData(kind, data) {
  return Array.isArray(data) ? { __kind: kind, fields: data } : { __kind: kind, ...data ?? {} };
}
function isConcurrentMerkleTreeHeaderData(kind, value) {
  return value.__kind === kind;
}
function getConcurrentMerkleTreeHeaderDataV1Encoder() {
  return web3_js.getStructEncoder([
    ["maxBufferSize", web3_js.getU32Encoder()],
    ["maxDepth", web3_js.getU32Encoder()],
    ["authority", web3_js.getAddressEncoder()],
    ["creationSlot", web3_js.getU64Encoder()],
    ["padding", web3_js.getArrayEncoder(web3_js.getU8Encoder(), { size: 6 })]
  ]);
}
function getConcurrentMerkleTreeHeaderDataV1Decoder() {
  return web3_js.getStructDecoder([
    ["maxBufferSize", web3_js.getU32Decoder()],
    ["maxDepth", web3_js.getU32Decoder()],
    ["authority", web3_js.getAddressDecoder()],
    ["creationSlot", web3_js.getU64Decoder()],
    ["padding", web3_js.getArrayDecoder(web3_js.getU8Decoder(), { size: 6 })]
  ]);
}
function getConcurrentMerkleTreeHeaderDataV1Codec() {
  return web3_js.combineCodec(
    getConcurrentMerkleTreeHeaderDataV1Encoder(),
    getConcurrentMerkleTreeHeaderDataV1Decoder()
  );
}
function getPathNodeEncoder() {
  return web3_js.getStructEncoder([
    ["node", web3_js.fixEncoderSize(web3_js.getBytesEncoder(), 32)],
    ["index", web3_js.getU32Encoder()]
  ]);
}
function getPathNodeDecoder() {
  return web3_js.getStructDecoder([
    ["node", web3_js.fixDecoderSize(web3_js.getBytesDecoder(), 32)],
    ["index", web3_js.getU32Decoder()]
  ]);
}
function getPathNodeCodec() {
  return web3_js.combineCodec(getPathNodeEncoder(), getPathNodeDecoder());
}
var BubblegumEventType = /* @__PURE__ */ ((BubblegumEventType2) => {
  BubblegumEventType2[BubblegumEventType2["Uninitialized"] = 0] = "Uninitialized";
  BubblegumEventType2[BubblegumEventType2["LeafSchemaEvent"] = 1] = "LeafSchemaEvent";
  return BubblegumEventType2;
})(BubblegumEventType || {});
function getBubblegumEventTypeEncoder() {
  return web3_js.getEnumEncoder(BubblegumEventType);
}
function getBubblegumEventTypeDecoder() {
  return web3_js.getEnumDecoder(BubblegumEventType);
}
function getBubblegumEventTypeCodec() {
  return web3_js.combineCodec(
    getBubblegumEventTypeEncoder(),
    getBubblegumEventTypeDecoder()
  );
}
function getCollectionEncoder() {
  return web3_js.getStructEncoder([
    ["verified", web3_js.getBooleanEncoder()],
    ["key", web3_js.getAddressEncoder()]
  ]);
}
function getCollectionDecoder() {
  return web3_js.getStructDecoder([
    ["verified", web3_js.getBooleanDecoder()],
    ["key", web3_js.getAddressDecoder()]
  ]);
}
function getCollectionCodec() {
  return web3_js.combineCodec(getCollectionEncoder(), getCollectionDecoder());
}
function getCreatorEncoder() {
  return web3_js.getStructEncoder([
    ["address", web3_js.getAddressEncoder()],
    ["verified", web3_js.getBooleanEncoder()],
    ["share", web3_js.getU8Encoder()]
  ]);
}
function getCreatorDecoder() {
  return web3_js.getStructDecoder([
    ["address", web3_js.getAddressDecoder()],
    ["verified", web3_js.getBooleanDecoder()],
    ["share", web3_js.getU8Decoder()]
  ]);
}
function getCreatorCodec() {
  return web3_js.combineCodec(getCreatorEncoder(), getCreatorDecoder());
}
var DecompressibleState = /* @__PURE__ */ ((DecompressibleState5) => {
  DecompressibleState5[DecompressibleState5["Enabled"] = 0] = "Enabled";
  DecompressibleState5[DecompressibleState5["Disabled"] = 1] = "Disabled";
  return DecompressibleState5;
})(DecompressibleState || {});
function getDecompressibleStateEncoder() {
  return web3_js.getEnumEncoder(DecompressibleState);
}
function getDecompressibleStateDecoder() {
  return web3_js.getEnumDecoder(DecompressibleState);
}
function getDecompressibleStateCodec() {
  return web3_js.combineCodec(
    getDecompressibleStateEncoder(),
    getDecompressibleStateDecoder()
  );
}
var InstructionName = /* @__PURE__ */ ((InstructionName2) => {
  InstructionName2[InstructionName2["Unknown"] = 0] = "Unknown";
  InstructionName2[InstructionName2["MintV1"] = 1] = "MintV1";
  InstructionName2[InstructionName2["Redeem"] = 2] = "Redeem";
  InstructionName2[InstructionName2["CancelRedeem"] = 3] = "CancelRedeem";
  InstructionName2[InstructionName2["Transfer"] = 4] = "Transfer";
  InstructionName2[InstructionName2["Delegate"] = 5] = "Delegate";
  InstructionName2[InstructionName2["DecompressV1"] = 6] = "DecompressV1";
  InstructionName2[InstructionName2["Compress"] = 7] = "Compress";
  InstructionName2[InstructionName2["Burn"] = 8] = "Burn";
  InstructionName2[InstructionName2["CreateTree"] = 9] = "CreateTree";
  InstructionName2[InstructionName2["VerifyCreator"] = 10] = "VerifyCreator";
  InstructionName2[InstructionName2["UnverifyCreator"] = 11] = "UnverifyCreator";
  InstructionName2[InstructionName2["VerifyCollection"] = 12] = "VerifyCollection";
  InstructionName2[InstructionName2["UnverifyCollection"] = 13] = "UnverifyCollection";
  InstructionName2[InstructionName2["SetAndVerifyCollection"] = 14] = "SetAndVerifyCollection";
  InstructionName2[InstructionName2["MintToCollectionV1"] = 15] = "MintToCollectionV1";
  InstructionName2[InstructionName2["SetDecompressibleState"] = 16] = "SetDecompressibleState";
  InstructionName2[InstructionName2["UpdateMetadata"] = 17] = "UpdateMetadata";
  return InstructionName2;
})(InstructionName || {});
function getInstructionNameEncoder() {
  return web3_js.getEnumEncoder(InstructionName);
}
function getInstructionNameDecoder() {
  return web3_js.getEnumDecoder(InstructionName);
}
function getInstructionNameCodec() {
  return web3_js.combineCodec(getInstructionNameEncoder(), getInstructionNameDecoder());
}
function getLeafSchemaEncoder() {
  return web3_js.getDiscriminatedUnionEncoder([
    [
      "V1",
      web3_js.getStructEncoder([
        ["id", web3_js.getAddressEncoder()],
        ["owner", web3_js.getAddressEncoder()],
        ["delegate", web3_js.getAddressEncoder()],
        ["nonce", web3_js.getU64Encoder()],
        ["dataHash", web3_js.fixEncoderSize(web3_js.getBytesEncoder(), 32)],
        ["creatorHash", web3_js.fixEncoderSize(web3_js.getBytesEncoder(), 32)]
      ])
    ]
  ]);
}
function getLeafSchemaDecoder() {
  return web3_js.getDiscriminatedUnionDecoder([
    [
      "V1",
      web3_js.getStructDecoder([
        ["id", web3_js.getAddressDecoder()],
        ["owner", web3_js.getAddressDecoder()],
        ["delegate", web3_js.getAddressDecoder()],
        ["nonce", web3_js.getU64Decoder()],
        ["dataHash", web3_js.fixDecoderSize(web3_js.getBytesDecoder(), 32)],
        ["creatorHash", web3_js.fixDecoderSize(web3_js.getBytesDecoder(), 32)]
      ])
    ]
  ]);
}
function getLeafSchemaCodec() {
  return web3_js.combineCodec(getLeafSchemaEncoder(), getLeafSchemaDecoder());
}
function leafSchema(kind, data) {
  return Array.isArray(data) ? { __kind: kind, fields: data } : { __kind: kind, ...data ?? {} };
}
function isLeafSchema(kind, value) {
  return value.__kind === kind;
}
function getMetadataArgsEncoder() {
  return web3_js.getStructEncoder([
    ["name", web3_js.addEncoderSizePrefix(web3_js.getUtf8Encoder(), web3_js.getU32Encoder())],
    ["symbol", web3_js.addEncoderSizePrefix(web3_js.getUtf8Encoder(), web3_js.getU32Encoder())],
    ["uri", web3_js.addEncoderSizePrefix(web3_js.getUtf8Encoder(), web3_js.getU32Encoder())],
    ["sellerFeeBasisPoints", web3_js.getU16Encoder()],
    ["primarySaleHappened", web3_js.getBooleanEncoder()],
    ["isMutable", web3_js.getBooleanEncoder()],
    ["editionNonce", web3_js.getOptionEncoder(web3_js.getU8Encoder())],
    ["tokenStandard", web3_js.getOptionEncoder(getTokenStandardEncoder())],
    ["collection", web3_js.getOptionEncoder(getCollectionEncoder())],
    ["uses", web3_js.getOptionEncoder(getUsesEncoder())],
    ["tokenProgramVersion", getTokenProgramVersionEncoder()],
    ["creators", web3_js.getArrayEncoder(getCreatorEncoder())]
  ]);
}
function getMetadataArgsDecoder() {
  return web3_js.getStructDecoder([
    ["name", web3_js.addDecoderSizePrefix(web3_js.getUtf8Decoder(), web3_js.getU32Decoder())],
    ["symbol", web3_js.addDecoderSizePrefix(web3_js.getUtf8Decoder(), web3_js.getU32Decoder())],
    ["uri", web3_js.addDecoderSizePrefix(web3_js.getUtf8Decoder(), web3_js.getU32Decoder())],
    ["sellerFeeBasisPoints", web3_js.getU16Decoder()],
    ["primarySaleHappened", web3_js.getBooleanDecoder()],
    ["isMutable", web3_js.getBooleanDecoder()],
    ["editionNonce", web3_js.getOptionDecoder(web3_js.getU8Decoder())],
    ["tokenStandard", web3_js.getOptionDecoder(getTokenStandardDecoder())],
    ["collection", web3_js.getOptionDecoder(getCollectionDecoder())],
    ["uses", web3_js.getOptionDecoder(getUsesDecoder())],
    ["tokenProgramVersion", getTokenProgramVersionDecoder()],
    ["creators", web3_js.getArrayDecoder(getCreatorDecoder())]
  ]);
}
function getMetadataArgsCodec() {
  return web3_js.combineCodec(getMetadataArgsEncoder(), getMetadataArgsDecoder());
}
var TokenProgramVersion2 = /* @__PURE__ */ ((TokenProgramVersion3) => {
  TokenProgramVersion3[TokenProgramVersion3["Original"] = 0] = "Original";
  TokenProgramVersion3[TokenProgramVersion3["Token2022"] = 1] = "Token2022";
  return TokenProgramVersion3;
})(TokenProgramVersion2 || {});
function getTokenProgramVersionEncoder() {
  return web3_js.getEnumEncoder(TokenProgramVersion2);
}
function getTokenProgramVersionDecoder() {
  return web3_js.getEnumDecoder(TokenProgramVersion2);
}
function getTokenProgramVersionCodec() {
  return web3_js.combineCodec(
    getTokenProgramVersionEncoder(),
    getTokenProgramVersionDecoder()
  );
}
var TokenStandard2 = /* @__PURE__ */ ((TokenStandard3) => {
  TokenStandard3[TokenStandard3["NonFungible"] = 0] = "NonFungible";
  TokenStandard3[TokenStandard3["FungibleAsset"] = 1] = "FungibleAsset";
  TokenStandard3[TokenStandard3["Fungible"] = 2] = "Fungible";
  TokenStandard3[TokenStandard3["NonFungibleEdition"] = 3] = "NonFungibleEdition";
  return TokenStandard3;
})(TokenStandard2 || {});
function getTokenStandardEncoder() {
  return web3_js.getEnumEncoder(TokenStandard2);
}
function getTokenStandardDecoder() {
  return web3_js.getEnumDecoder(TokenStandard2);
}
function getTokenStandardCodec() {
  return web3_js.combineCodec(getTokenStandardEncoder(), getTokenStandardDecoder());
}
var UseMethod = /* @__PURE__ */ ((UseMethod3) => {
  UseMethod3[UseMethod3["Burn"] = 0] = "Burn";
  UseMethod3[UseMethod3["Multiple"] = 1] = "Multiple";
  UseMethod3[UseMethod3["Single"] = 2] = "Single";
  return UseMethod3;
})(UseMethod || {});
function getUseMethodEncoder() {
  return web3_js.getEnumEncoder(UseMethod);
}
function getUseMethodDecoder() {
  return web3_js.getEnumDecoder(UseMethod);
}
function getUseMethodCodec() {
  return web3_js.combineCodec(getUseMethodEncoder(), getUseMethodDecoder());
}
function getUsesEncoder() {
  return web3_js.getStructEncoder([
    ["useMethod", getUseMethodEncoder()],
    ["remaining", web3_js.getU64Encoder()],
    ["total", web3_js.getU64Encoder()]
  ]);
}
function getUsesDecoder() {
  return web3_js.getStructDecoder([
    ["useMethod", getUseMethodDecoder()],
    ["remaining", web3_js.getU64Decoder()],
    ["total", web3_js.getU64Decoder()]
  ]);
}
function getUsesCodec() {
  return web3_js.combineCodec(getUsesEncoder(), getUsesDecoder());
}
var Version = /* @__PURE__ */ ((Version2) => {
  Version2[Version2["V1"] = 0] = "V1";
  return Version2;
})(Version || {});
function getVersionEncoder() {
  return web3_js.getEnumEncoder(Version);
}
function getVersionDecoder() {
  return web3_js.getEnumDecoder(Version);
}
function getVersionCodec() {
  return web3_js.combineCodec(getVersionEncoder(), getVersionDecoder());
}

// src/generated/accounts/treeConfig.ts
function getTreeConfigEncoder() {
  return web3_js.transformEncoder(
    web3_js.getStructEncoder([
      ["discriminator", web3_js.fixEncoderSize(web3_js.getBytesEncoder(), 8)],
      ["treeCreator", web3_js.getAddressEncoder()],
      ["treeDelegate", web3_js.getAddressEncoder()],
      ["totalMintCapacity", web3_js.getU64Encoder()],
      ["numMinted", web3_js.getU64Encoder()],
      ["isPublic", web3_js.getBooleanEncoder()],
      ["isDecompressible", getDecompressibleStateEncoder()]
    ]),
    (value) => ({
      ...value,
      discriminator: new Uint8Array([122, 245, 175, 248, 171, 34, 0, 207])
    })
  );
}
function getTreeConfigDecoder() {
  return web3_js.getStructDecoder([
    ["discriminator", web3_js.fixDecoderSize(web3_js.getBytesDecoder(), 8)],
    ["treeCreator", web3_js.getAddressDecoder()],
    ["treeDelegate", web3_js.getAddressDecoder()],
    ["totalMintCapacity", web3_js.getU64Decoder()],
    ["numMinted", web3_js.getU64Decoder()],
    ["isPublic", web3_js.getBooleanDecoder()],
    ["isDecompressible", getDecompressibleStateDecoder()]
  ]);
}
function getTreeConfigCodec() {
  return web3_js.combineCodec(getTreeConfigEncoder(), getTreeConfigDecoder());
}
function decodeTreeConfig(encodedAccount) {
  return web3_js.decodeAccount(
    encodedAccount,
    getTreeConfigDecoder()
  );
}
async function fetchTreeConfig(rpc, address, config) {
  const maybeAccount = await fetchMaybeTreeConfig(rpc, address, config);
  web3_js.assertAccountExists(maybeAccount);
  return maybeAccount;
}
async function fetchMaybeTreeConfig(rpc, address, config) {
  const maybeAccount = await web3_js.fetchEncodedAccount(rpc, address, config);
  return decodeTreeConfig(maybeAccount);
}
async function fetchAllTreeConfig(rpc, addresses, config) {
  const maybeAccounts = await fetchAllMaybeTreeConfig(rpc, addresses, config);
  web3_js.assertAccountsExist(maybeAccounts);
  return maybeAccounts;
}
async function fetchAllMaybeTreeConfig(rpc, addresses, config) {
  const maybeAccounts = await web3_js.fetchEncodedAccounts(rpc, addresses, config);
  return maybeAccounts.map((maybeAccount) => decodeTreeConfig(maybeAccount));
}
function getTreeConfigSize() {
  return 90;
}
function getVoucherEncoder() {
  return web3_js.transformEncoder(
    web3_js.getStructEncoder([
      ["discriminator", web3_js.fixEncoderSize(web3_js.getBytesEncoder(), 8)],
      ["leafSchema", getLeafSchemaEncoder()],
      ["index", web3_js.getU32Encoder()],
      ["merkleTree", web3_js.getAddressEncoder()]
    ]),
    (value) => ({
      ...value,
      discriminator: new Uint8Array([191, 204, 149, 234, 213, 165, 13, 65])
    })
  );
}
function getVoucherDecoder() {
  return web3_js.getStructDecoder([
    ["discriminator", web3_js.fixDecoderSize(web3_js.getBytesDecoder(), 8)],
    ["leafSchema", getLeafSchemaDecoder()],
    ["index", web3_js.getU32Decoder()],
    ["merkleTree", web3_js.getAddressDecoder()]
  ]);
}
function getVoucherCodec() {
  return web3_js.combineCodec(getVoucherEncoder(), getVoucherDecoder());
}
function decodeVoucher(encodedAccount) {
  return web3_js.decodeAccount(
    encodedAccount,
    getVoucherDecoder()
  );
}
async function fetchVoucher(rpc, address, config) {
  const maybeAccount = await fetchMaybeVoucher(rpc, address, config);
  web3_js.assertAccountExists(maybeAccount);
  return maybeAccount;
}
async function fetchMaybeVoucher(rpc, address, config) {
  const maybeAccount = await web3_js.fetchEncodedAccount(rpc, address, config);
  return decodeVoucher(maybeAccount);
}
async function fetchAllVoucher(rpc, addresses, config) {
  const maybeAccounts = await fetchAllMaybeVoucher(rpc, addresses, config);
  web3_js.assertAccountsExist(maybeAccounts);
  return maybeAccounts;
}
async function fetchAllMaybeVoucher(rpc, addresses, config) {
  const maybeAccounts = await web3_js.fetchEncodedAccounts(rpc, addresses, config);
  return maybeAccounts.map((maybeAccount) => decodeVoucher(maybeAccount));
}
function getVoucherSize() {
  return 213;
}

// src/generated/errors/bubblegum.ts
var BUBBLEGUM_ERROR__ASSET_OWNER_MISMATCH = 6e3;
var BUBBLEGUM_ERROR__PUBLIC_KEY_MISMATCH = 6001;
var BUBBLEGUM_ERROR__HASHING_MISMATCH = 6002;
var BUBBLEGUM_ERROR__UNSUPPORTED_SCHEMA_VERSION = 6003;
var BUBBLEGUM_ERROR__CREATOR_SHARE_TOTAL_MUST_BE100 = 6004;
var BUBBLEGUM_ERROR__DUPLICATE_CREATOR_ADDRESS = 6005;
var BUBBLEGUM_ERROR__CREATOR_DID_NOT_VERIFY = 6006;
var BUBBLEGUM_ERROR__CREATOR_NOT_FOUND = 6007;
var BUBBLEGUM_ERROR__NO_CREATORS_PRESENT = 6008;
var BUBBLEGUM_ERROR__CREATOR_HASH_MISMATCH = 6009;
var BUBBLEGUM_ERROR__DATA_HASH_MISMATCH = 6010;
var BUBBLEGUM_ERROR__CREATORS_TOO_LONG = 6011;
var BUBBLEGUM_ERROR__METADATA_NAME_TOO_LONG = 6012;
var BUBBLEGUM_ERROR__METADATA_SYMBOL_TOO_LONG = 6013;
var BUBBLEGUM_ERROR__METADATA_URI_TOO_LONG = 6014;
var BUBBLEGUM_ERROR__METADATA_BASIS_POINTS_TOO_HIGH = 6015;
var BUBBLEGUM_ERROR__TREE_AUTHORITY_INCORRECT = 6016;
var BUBBLEGUM_ERROR__INSUFFICIENT_MINT_CAPACITY = 6017;
var BUBBLEGUM_ERROR__NUMERICAL_OVERFLOW_ERROR = 6018;
var BUBBLEGUM_ERROR__INCORRECT_OWNER = 6019;
var BUBBLEGUM_ERROR__COLLECTION_CANNOT_BE_VERIFIED_IN_THIS_INSTRUCTION = 6020;
var BUBBLEGUM_ERROR__COLLECTION_NOT_FOUND = 6021;
var BUBBLEGUM_ERROR__ALREADY_VERIFIED = 6022;
var BUBBLEGUM_ERROR__ALREADY_UNVERIFIED = 6023;
var BUBBLEGUM_ERROR__UPDATE_AUTHORITY_INCORRECT = 6024;
var BUBBLEGUM_ERROR__LEAF_AUTHORITY_MUST_SIGN = 6025;
var BUBBLEGUM_ERROR__COLLECTION_MUST_BE_SIZED = 6026;
var BUBBLEGUM_ERROR__METADATA_MINT_MISMATCH = 6027;
var BUBBLEGUM_ERROR__INVALID_COLLECTION_AUTHORITY = 6028;
var BUBBLEGUM_ERROR__INVALID_DELEGATE_RECORD = 6029;
var BUBBLEGUM_ERROR__COLLECTION_MASTER_EDITION_ACCOUNT_INVALID = 6030;
var BUBBLEGUM_ERROR__COLLECTION_MUST_BE_A_UNIQUE_MASTER_EDITION = 6031;
var BUBBLEGUM_ERROR__UNKNOWN_EXTERNAL_ERROR = 6032;
var BUBBLEGUM_ERROR__DECOMPRESSION_DISABLED = 6033;
var BUBBLEGUM_ERROR__MISSING_COLLECTION_MINT_ACCOUNT = 6034;
var BUBBLEGUM_ERROR__MISSING_COLLECTION_METADATA_ACCOUNT = 6035;
var BUBBLEGUM_ERROR__COLLECTION_MISMATCH = 6036;
var BUBBLEGUM_ERROR__METADATA_IMMUTABLE = 6037;
var BUBBLEGUM_ERROR__PRIMARY_SALE_CAN_ONLY_BE_FLIPPED_TO_TRUE = 6038;
var BUBBLEGUM_ERROR__CREATOR_DID_NOT_UNVERIFY = 6039;
var BUBBLEGUM_ERROR__INVALID_TOKEN_STANDARD = 6040;
var BUBBLEGUM_ERROR__INVALID_CANOPY_SIZE = 6041;
var bubblegumErrorMessages;
if (__DEV__) {
  bubblegumErrorMessages = {
    [BUBBLEGUM_ERROR__ALREADY_UNVERIFIED]: `Collection item is already unverified.`,
    [BUBBLEGUM_ERROR__ALREADY_VERIFIED]: `Collection item is already verified.`,
    [BUBBLEGUM_ERROR__ASSET_OWNER_MISMATCH]: `Asset Owner Does not match`,
    [BUBBLEGUM_ERROR__COLLECTION_CANNOT_BE_VERIFIED_IN_THIS_INSTRUCTION]: `Cannot Verify Collection in this Instruction`,
    [BUBBLEGUM_ERROR__COLLECTION_MASTER_EDITION_ACCOUNT_INVALID]: `Edition account doesnt match collection`,
    [BUBBLEGUM_ERROR__COLLECTION_MISMATCH]: `Collection mismatch`,
    [BUBBLEGUM_ERROR__COLLECTION_MUST_BE_A_UNIQUE_MASTER_EDITION]: `Collection Must Be a Unique Master Edition v2`,
    [BUBBLEGUM_ERROR__COLLECTION_MUST_BE_SIZED]: `Collection Not Compatable with Compression, Must be Sized`,
    [BUBBLEGUM_ERROR__COLLECTION_NOT_FOUND]: `Collection Not Found on Metadata`,
    [BUBBLEGUM_ERROR__CREATOR_DID_NOT_UNVERIFY]: `Creator did not unverify the metadata`,
    [BUBBLEGUM_ERROR__CREATOR_DID_NOT_VERIFY]: `Creator did not verify the metadata`,
    [BUBBLEGUM_ERROR__CREATOR_HASH_MISMATCH]: `User-provided creator Vec must result in same user-provided creator hash`,
    [BUBBLEGUM_ERROR__CREATOR_NOT_FOUND]: `Creator not found in creator Vec`,
    [BUBBLEGUM_ERROR__CREATOR_SHARE_TOTAL_MUST_BE100]: `Creator shares must sum to 100`,
    [BUBBLEGUM_ERROR__CREATORS_TOO_LONG]: `Creators list too long`,
    [BUBBLEGUM_ERROR__DATA_HASH_MISMATCH]: `User-provided metadata must result in same user-provided data hash`,
    [BUBBLEGUM_ERROR__DECOMPRESSION_DISABLED]: `Decompression is disabled for this tree.`,
    [BUBBLEGUM_ERROR__DUPLICATE_CREATOR_ADDRESS]: `No duplicate creator addresses in metadata`,
    [BUBBLEGUM_ERROR__HASHING_MISMATCH]: `Hashing Mismatch Within Leaf Schema`,
    [BUBBLEGUM_ERROR__INCORRECT_OWNER]: `Incorrect account owner`,
    [BUBBLEGUM_ERROR__INSUFFICIENT_MINT_CAPACITY]: `Not enough unapproved mints left`,
    [BUBBLEGUM_ERROR__INVALID_CANOPY_SIZE]: `Canopy size should be set bigger for this tree`,
    [BUBBLEGUM_ERROR__INVALID_COLLECTION_AUTHORITY]: `Invalid collection authority`,
    [BUBBLEGUM_ERROR__INVALID_DELEGATE_RECORD]: `Invalid delegate record pda derivation`,
    [BUBBLEGUM_ERROR__INVALID_TOKEN_STANDARD]: `Only NonFungible standard is supported`,
    [BUBBLEGUM_ERROR__LEAF_AUTHORITY_MUST_SIGN]: `This transaction must be signed by either the leaf owner or leaf delegate`,
    [BUBBLEGUM_ERROR__METADATA_BASIS_POINTS_TOO_HIGH]: `Basis points in metadata cannot exceed 10000`,
    [BUBBLEGUM_ERROR__METADATA_IMMUTABLE]: `Metadata not mutable`,
    [BUBBLEGUM_ERROR__METADATA_MINT_MISMATCH]: `Metadata mint does not match collection mint`,
    [BUBBLEGUM_ERROR__METADATA_NAME_TOO_LONG]: `Name in metadata is too long`,
    [BUBBLEGUM_ERROR__METADATA_SYMBOL_TOO_LONG]: `Symbol in metadata is too long`,
    [BUBBLEGUM_ERROR__METADATA_URI_TOO_LONG]: `Uri in metadata is too long`,
    [BUBBLEGUM_ERROR__MISSING_COLLECTION_METADATA_ACCOUNT]: `Missing collection metadata account`,
    [BUBBLEGUM_ERROR__MISSING_COLLECTION_MINT_ACCOUNT]: `Missing collection mint account`,
    [BUBBLEGUM_ERROR__NO_CREATORS_PRESENT]: `No creators in creator Vec`,
    [BUBBLEGUM_ERROR__NUMERICAL_OVERFLOW_ERROR]: `NumericalOverflowError`,
    [BUBBLEGUM_ERROR__PRIMARY_SALE_CAN_ONLY_BE_FLIPPED_TO_TRUE]: `Can only update primary sale to true`,
    [BUBBLEGUM_ERROR__PUBLIC_KEY_MISMATCH]: `PublicKeyMismatch`,
    [BUBBLEGUM_ERROR__TREE_AUTHORITY_INCORRECT]: `Tree creator or tree delegate must sign.`,
    [BUBBLEGUM_ERROR__UNKNOWN_EXTERNAL_ERROR]: `Could not convert external error to BubblegumError`,
    [BUBBLEGUM_ERROR__UNSUPPORTED_SCHEMA_VERSION]: `Unsupported Schema Version`,
    [BUBBLEGUM_ERROR__UPDATE_AUTHORITY_INCORRECT]: `Incorrect leaf metadata update authority.`
  };
}
function getBubblegumErrorMessage(code) {
  if (__DEV__) {
    return bubblegumErrorMessages[code];
  }
  return "Error message not available in production bundles. Compile with `__DEV__` set to true to see more information.";
}
var BUBBLEGUM_PROGRAM_ADDRESS = "BGUMAp9Gq7iTEuizy4pqaxsTyUCBK68MDfK752saRPUY";
var BubblegumAccount = /* @__PURE__ */ ((BubblegumAccount2) => {
  BubblegumAccount2[BubblegumAccount2["TreeConfig"] = 0] = "TreeConfig";
  BubblegumAccount2[BubblegumAccount2["Voucher"] = 1] = "Voucher";
  return BubblegumAccount2;
})(BubblegumAccount || {});
function identifyBubblegumAccount(account) {
  const data = account instanceof Uint8Array ? account : account.data;
  if (web3_js.containsBytes(
    data,
    web3_js.fixEncoderSize(web3_js.getBytesEncoder(), 8).encode(
      new Uint8Array([122, 245, 175, 248, 171, 34, 0, 207])
    ),
    0
  )) {
    return 0 /* TreeConfig */;
  }
  if (web3_js.containsBytes(
    data,
    web3_js.fixEncoderSize(web3_js.getBytesEncoder(), 8).encode(
      new Uint8Array([191, 204, 149, 234, 213, 165, 13, 65])
    ),
    0
  )) {
    return 1 /* Voucher */;
  }
  throw new Error(
    "The provided account could not be identified as a bubblegum account."
  );
}
var BubblegumInstruction = /* @__PURE__ */ ((BubblegumInstruction2) => {
  BubblegumInstruction2[BubblegumInstruction2["Burn"] = 0] = "Burn";
  BubblegumInstruction2[BubblegumInstruction2["CancelRedeem"] = 1] = "CancelRedeem";
  BubblegumInstruction2[BubblegumInstruction2["Compress"] = 2] = "Compress";
  BubblegumInstruction2[BubblegumInstruction2["CreateTree"] = 3] = "CreateTree";
  BubblegumInstruction2[BubblegumInstruction2["DecompressV1"] = 4] = "DecompressV1";
  BubblegumInstruction2[BubblegumInstruction2["Delegate"] = 5] = "Delegate";
  BubblegumInstruction2[BubblegumInstruction2["MintToCollectionV1"] = 6] = "MintToCollectionV1";
  BubblegumInstruction2[BubblegumInstruction2["MintV1"] = 7] = "MintV1";
  BubblegumInstruction2[BubblegumInstruction2["Redeem"] = 8] = "Redeem";
  BubblegumInstruction2[BubblegumInstruction2["SetAndVerifyCollection"] = 9] = "SetAndVerifyCollection";
  BubblegumInstruction2[BubblegumInstruction2["SetDecompressableState"] = 10] = "SetDecompressableState";
  BubblegumInstruction2[BubblegumInstruction2["SetDecompressibleState"] = 11] = "SetDecompressibleState";
  BubblegumInstruction2[BubblegumInstruction2["SetTreeDelegate"] = 12] = "SetTreeDelegate";
  BubblegumInstruction2[BubblegumInstruction2["Transfer"] = 13] = "Transfer";
  BubblegumInstruction2[BubblegumInstruction2["UnverifyCollection"] = 14] = "UnverifyCollection";
  BubblegumInstruction2[BubblegumInstruction2["UnverifyCreator"] = 15] = "UnverifyCreator";
  BubblegumInstruction2[BubblegumInstruction2["VerifyCollection"] = 16] = "VerifyCollection";
  BubblegumInstruction2[BubblegumInstruction2["VerifyCreator"] = 17] = "VerifyCreator";
  BubblegumInstruction2[BubblegumInstruction2["UpdateMetadata"] = 18] = "UpdateMetadata";
  return BubblegumInstruction2;
})(BubblegumInstruction || {});
function identifyBubblegumInstruction(instruction) {
  const data = instruction instanceof Uint8Array ? instruction : instruction.data;
  if (web3_js.containsBytes(
    data,
    web3_js.fixEncoderSize(web3_js.getBytesEncoder(), 8).encode(
      new Uint8Array([116, 110, 29, 56, 107, 219, 42, 93])
    ),
    0
  )) {
    return 0 /* Burn */;
  }
  if (web3_js.containsBytes(
    data,
    web3_js.fixEncoderSize(web3_js.getBytesEncoder(), 8).encode(
      new Uint8Array([111, 76, 232, 50, 39, 175, 48, 242])
    ),
    0
  )) {
    return 1 /* CancelRedeem */;
  }
  if (web3_js.containsBytes(
    data,
    web3_js.fixEncoderSize(web3_js.getBytesEncoder(), 8).encode(
      new Uint8Array([82, 193, 176, 117, 176, 21, 115, 253])
    ),
    0
  )) {
    return 2 /* Compress */;
  }
  if (web3_js.containsBytes(
    data,
    web3_js.fixEncoderSize(web3_js.getBytesEncoder(), 8).encode(
      new Uint8Array([165, 83, 136, 142, 89, 202, 47, 220])
    ),
    0
  )) {
    return 3 /* CreateTree */;
  }
  if (web3_js.containsBytes(
    data,
    web3_js.fixEncoderSize(web3_js.getBytesEncoder(), 8).encode(
      new Uint8Array([54, 85, 76, 70, 228, 250, 164, 81])
    ),
    0
  )) {
    return 4 /* DecompressV1 */;
  }
  if (web3_js.containsBytes(
    data,
    web3_js.fixEncoderSize(web3_js.getBytesEncoder(), 8).encode(
      new Uint8Array([90, 147, 75, 178, 85, 88, 4, 137])
    ),
    0
  )) {
    return 5 /* Delegate */;
  }
  if (web3_js.containsBytes(
    data,
    web3_js.fixEncoderSize(web3_js.getBytesEncoder(), 8).encode(
      new Uint8Array([153, 18, 178, 47, 197, 158, 86, 15])
    ),
    0
  )) {
    return 6 /* MintToCollectionV1 */;
  }
  if (web3_js.containsBytes(
    data,
    web3_js.fixEncoderSize(web3_js.getBytesEncoder(), 8).encode(
      new Uint8Array([145, 98, 192, 118, 184, 147, 118, 104])
    ),
    0
  )) {
    return 7 /* MintV1 */;
  }
  if (web3_js.containsBytes(
    data,
    web3_js.fixEncoderSize(web3_js.getBytesEncoder(), 8).encode(
      new Uint8Array([184, 12, 86, 149, 70, 196, 97, 225])
    ),
    0
  )) {
    return 8 /* Redeem */;
  }
  if (web3_js.containsBytes(
    data,
    web3_js.fixEncoderSize(web3_js.getBytesEncoder(), 8).encode(
      new Uint8Array([235, 242, 121, 216, 158, 234, 180, 234])
    ),
    0
  )) {
    return 9 /* SetAndVerifyCollection */;
  }
  if (web3_js.containsBytes(
    data,
    web3_js.fixEncoderSize(web3_js.getBytesEncoder(), 8).encode(
      new Uint8Array([18, 135, 238, 168, 246, 195, 61, 115])
    ),
    0
  )) {
    return 10 /* SetDecompressableState */;
  }
  if (web3_js.containsBytes(
    data,
    web3_js.fixEncoderSize(web3_js.getBytesEncoder(), 8).encode(
      new Uint8Array([82, 104, 152, 6, 149, 111, 100, 13])
    ),
    0
  )) {
    return 11 /* SetDecompressibleState */;
  }
  if (web3_js.containsBytes(
    data,
    web3_js.fixEncoderSize(web3_js.getBytesEncoder(), 8).encode(
      new Uint8Array([253, 118, 66, 37, 190, 49, 154, 102])
    ),
    0
  )) {
    return 12 /* SetTreeDelegate */;
  }
  if (web3_js.containsBytes(
    data,
    web3_js.fixEncoderSize(web3_js.getBytesEncoder(), 8).encode(
      new Uint8Array([163, 52, 200, 231, 140, 3, 69, 186])
    ),
    0
  )) {
    return 13 /* Transfer */;
  }
  if (web3_js.containsBytes(
    data,
    web3_js.fixEncoderSize(web3_js.getBytesEncoder(), 8).encode(
      new Uint8Array([250, 251, 42, 106, 41, 137, 186, 168])
    ),
    0
  )) {
    return 14 /* UnverifyCollection */;
  }
  if (web3_js.containsBytes(
    data,
    web3_js.fixEncoderSize(web3_js.getBytesEncoder(), 8).encode(
      new Uint8Array([107, 178, 57, 39, 105, 115, 112, 152])
    ),
    0
  )) {
    return 15 /* UnverifyCreator */;
  }
  if (web3_js.containsBytes(
    data,
    web3_js.fixEncoderSize(web3_js.getBytesEncoder(), 8).encode(
      new Uint8Array([56, 113, 101, 253, 79, 55, 122, 169])
    ),
    0
  )) {
    return 16 /* VerifyCollection */;
  }
  if (web3_js.containsBytes(
    data,
    web3_js.fixEncoderSize(web3_js.getBytesEncoder(), 8).encode(
      new Uint8Array([52, 17, 96, 132, 71, 4, 85, 194])
    ),
    0
  )) {
    return 17 /* VerifyCreator */;
  }
  if (web3_js.containsBytes(
    data,
    web3_js.fixEncoderSize(web3_js.getBytesEncoder(), 8).encode(
      new Uint8Array([170, 182, 43, 239, 97, 78, 225, 186])
    ),
    0
  )) {
    return 18 /* UpdateMetadata */;
  }
  throw new Error(
    "The provided instruction could not be identified as a bubblegum instruction."
  );
}
function expectAddress2(value) {
  if (!value) {
    throw new Error("Expected a Address.");
  }
  if (typeof value === "object" && "address" in value) {
    return value.address;
  }
  if (Array.isArray(value)) {
    return value[0];
  }
  return value;
}
function getAccountMetaFactory2(programAddress, optionalAccountStrategy) {
  return (account) => {
    if (!account.value) {
      return Object.freeze({
        address: programAddress,
        role: web3_js.AccountRole.READONLY
      });
    }
    const writableRole = account.isWritable ? web3_js.AccountRole.WRITABLE : web3_js.AccountRole.READONLY;
    return Object.freeze({
      address: expectAddress2(account.value),
      role: isTransactionSigner2(account.value) ? web3_js.upgradeRoleToSigner(writableRole) : writableRole,
      ...isTransactionSigner2(account.value) ? { signer: account.value } : {}
    });
  };
}
function isTransactionSigner2(value) {
  return !!value && typeof value === "object" && "address" in value && web3_js.isTransactionSigner(value);
}

// src/generated/instructions/burn.ts
function getBurnInstructionDataEncoder() {
  return web3_js.transformEncoder(
    web3_js.getStructEncoder([
      ["discriminator", web3_js.fixEncoderSize(web3_js.getBytesEncoder(), 8)],
      ["root", web3_js.fixEncoderSize(web3_js.getBytesEncoder(), 32)],
      ["dataHash", web3_js.fixEncoderSize(web3_js.getBytesEncoder(), 32)],
      ["creatorHash", web3_js.fixEncoderSize(web3_js.getBytesEncoder(), 32)],
      ["nonce", web3_js.getU64Encoder()],
      ["index", web3_js.getU32Encoder()]
    ]),
    (value) => ({
      ...value,
      discriminator: new Uint8Array([116, 110, 29, 56, 107, 219, 42, 93])
    })
  );
}
function getBurnInstructionDataDecoder() {
  return web3_js.getStructDecoder([
    ["discriminator", web3_js.fixDecoderSize(web3_js.getBytesDecoder(), 8)],
    ["root", web3_js.fixDecoderSize(web3_js.getBytesDecoder(), 32)],
    ["dataHash", web3_js.fixDecoderSize(web3_js.getBytesDecoder(), 32)],
    ["creatorHash", web3_js.fixDecoderSize(web3_js.getBytesDecoder(), 32)],
    ["nonce", web3_js.getU64Decoder()],
    ["index", web3_js.getU32Decoder()]
  ]);
}
function getBurnInstructionDataCodec() {
  return web3_js.combineCodec(
    getBurnInstructionDataEncoder(),
    getBurnInstructionDataDecoder()
  );
}
function getBurnInstruction(input) {
  const programAddress = BUBBLEGUM_PROGRAM_ADDRESS;
  const originalAccounts = {
    treeAuthority: { value: input.treeAuthority ?? null, isWritable: false },
    leafOwner: { value: input.leafOwner ?? null, isWritable: false },
    leafDelegate: { value: input.leafDelegate ?? null, isWritable: false },
    merkleTree: { value: input.merkleTree ?? null, isWritable: true },
    logWrapper: { value: input.logWrapper ?? null, isWritable: false },
    compressionProgram: {
      value: input.compressionProgram ?? null,
      isWritable: false
    },
    systemProgram: { value: input.systemProgram ?? null, isWritable: false }
  };
  const accounts = originalAccounts;
  const args = { ...input };
  if (!accounts.systemProgram.value) {
    accounts.systemProgram.value = "11111111111111111111111111111111";
  }
  const getAccountMeta = getAccountMetaFactory2(programAddress);
  const instruction = {
    accounts: [
      getAccountMeta(accounts.treeAuthority),
      getAccountMeta(accounts.leafOwner),
      getAccountMeta(accounts.leafDelegate),
      getAccountMeta(accounts.merkleTree),
      getAccountMeta(accounts.logWrapper),
      getAccountMeta(accounts.compressionProgram),
      getAccountMeta(accounts.systemProgram)
    ],
    programAddress,
    data: getBurnInstructionDataEncoder().encode(
      args
    )
  };
  return instruction;
}
function parseBurnInstruction(instruction) {
  if (instruction.accounts.length < 7) {
    throw new Error("Not enough accounts");
  }
  let accountIndex = 0;
  const getNextAccount = () => {
    const accountMeta = instruction.accounts[accountIndex];
    accountIndex += 1;
    return accountMeta;
  };
  return {
    programAddress: instruction.programAddress,
    accounts: {
      treeAuthority: getNextAccount(),
      leafOwner: getNextAccount(),
      leafDelegate: getNextAccount(),
      merkleTree: getNextAccount(),
      logWrapper: getNextAccount(),
      compressionProgram: getNextAccount(),
      systemProgram: getNextAccount()
    },
    data: getBurnInstructionDataDecoder().decode(instruction.data)
  };
}
function getCancelRedeemInstructionDataEncoder() {
  return web3_js.transformEncoder(
    web3_js.getStructEncoder([
      ["discriminator", web3_js.fixEncoderSize(web3_js.getBytesEncoder(), 8)],
      ["root", web3_js.fixEncoderSize(web3_js.getBytesEncoder(), 32)]
    ]),
    (value) => ({
      ...value,
      discriminator: new Uint8Array([111, 76, 232, 50, 39, 175, 48, 242])
    })
  );
}
function getCancelRedeemInstructionDataDecoder() {
  return web3_js.getStructDecoder([
    ["discriminator", web3_js.fixDecoderSize(web3_js.getBytesDecoder(), 8)],
    ["root", web3_js.fixDecoderSize(web3_js.getBytesDecoder(), 32)]
  ]);
}
function getCancelRedeemInstructionDataCodec() {
  return web3_js.combineCodec(
    getCancelRedeemInstructionDataEncoder(),
    getCancelRedeemInstructionDataDecoder()
  );
}
function getCancelRedeemInstruction(input) {
  const programAddress = BUBBLEGUM_PROGRAM_ADDRESS;
  const originalAccounts = {
    treeAuthority: { value: input.treeAuthority ?? null, isWritable: false },
    leafOwner: { value: input.leafOwner ?? null, isWritable: true },
    merkleTree: { value: input.merkleTree ?? null, isWritable: true },
    voucher: { value: input.voucher ?? null, isWritable: true },
    logWrapper: { value: input.logWrapper ?? null, isWritable: false },
    compressionProgram: {
      value: input.compressionProgram ?? null,
      isWritable: false
    },
    systemProgram: { value: input.systemProgram ?? null, isWritable: false }
  };
  const accounts = originalAccounts;
  const args = { ...input };
  if (!accounts.systemProgram.value) {
    accounts.systemProgram.value = "11111111111111111111111111111111";
  }
  const getAccountMeta = getAccountMetaFactory2(programAddress);
  const instruction = {
    accounts: [
      getAccountMeta(accounts.treeAuthority),
      getAccountMeta(accounts.leafOwner),
      getAccountMeta(accounts.merkleTree),
      getAccountMeta(accounts.voucher),
      getAccountMeta(accounts.logWrapper),
      getAccountMeta(accounts.compressionProgram),
      getAccountMeta(accounts.systemProgram)
    ],
    programAddress,
    data: getCancelRedeemInstructionDataEncoder().encode(
      args
    )
  };
  return instruction;
}
function parseCancelRedeemInstruction(instruction) {
  if (instruction.accounts.length < 7) {
    throw new Error("Not enough accounts");
  }
  let accountIndex = 0;
  const getNextAccount = () => {
    const accountMeta = instruction.accounts[accountIndex];
    accountIndex += 1;
    return accountMeta;
  };
  return {
    programAddress: instruction.programAddress,
    accounts: {
      treeAuthority: getNextAccount(),
      leafOwner: getNextAccount(),
      merkleTree: getNextAccount(),
      voucher: getNextAccount(),
      logWrapper: getNextAccount(),
      compressionProgram: getNextAccount(),
      systemProgram: getNextAccount()
    },
    data: getCancelRedeemInstructionDataDecoder().decode(instruction.data)
  };
}
function getCompressInstructionDataEncoder() {
  return web3_js.transformEncoder(
    web3_js.getStructEncoder([["discriminator", web3_js.fixEncoderSize(web3_js.getBytesEncoder(), 8)]]),
    (value) => ({
      ...value,
      discriminator: new Uint8Array([82, 193, 176, 117, 176, 21, 115, 253])
    })
  );
}
function getCompressInstructionDataDecoder() {
  return web3_js.getStructDecoder([
    ["discriminator", web3_js.fixDecoderSize(web3_js.getBytesDecoder(), 8)]
  ]);
}
function getCompressInstructionDataCodec() {
  return web3_js.combineCodec(
    getCompressInstructionDataEncoder(),
    getCompressInstructionDataDecoder()
  );
}
function getCompressInstruction(input) {
  const programAddress = BUBBLEGUM_PROGRAM_ADDRESS;
  const originalAccounts = {
    treeAuthority: { value: input.treeAuthority ?? null, isWritable: false },
    leafOwner: { value: input.leafOwner ?? null, isWritable: false },
    leafDelegate: { value: input.leafDelegate ?? null, isWritable: false },
    merkleTree: { value: input.merkleTree ?? null, isWritable: false },
    tokenAccount: { value: input.tokenAccount ?? null, isWritable: true },
    mint: { value: input.mint ?? null, isWritable: true },
    metadata: { value: input.metadata ?? null, isWritable: true },
    masterEdition: { value: input.masterEdition ?? null, isWritable: true },
    payer: { value: input.payer ?? null, isWritable: true },
    logWrapper: { value: input.logWrapper ?? null, isWritable: false },
    compressionProgram: {
      value: input.compressionProgram ?? null,
      isWritable: false
    },
    tokenProgram: { value: input.tokenProgram ?? null, isWritable: false },
    tokenMetadataProgram: {
      value: input.tokenMetadataProgram ?? null,
      isWritable: false
    },
    systemProgram: { value: input.systemProgram ?? null, isWritable: false }
  };
  const accounts = originalAccounts;
  if (!accounts.tokenProgram.value) {
    accounts.tokenProgram.value = "TokenkegQfeZyiNwAJbNbGKPFXCWuBvf9Ss623VQ5DA";
  }
  if (!accounts.tokenMetadataProgram.value) {
    accounts.tokenMetadataProgram.value = "metaqbxxUerdq28cj1RbAWkYQm3ybzjb6a8bt518x1s";
  }
  if (!accounts.systemProgram.value) {
    accounts.systemProgram.value = "11111111111111111111111111111111";
  }
  const getAccountMeta = getAccountMetaFactory2(programAddress);
  const instruction = {
    accounts: [
      getAccountMeta(accounts.treeAuthority),
      getAccountMeta(accounts.leafOwner),
      getAccountMeta(accounts.leafDelegate),
      getAccountMeta(accounts.merkleTree),
      getAccountMeta(accounts.tokenAccount),
      getAccountMeta(accounts.mint),
      getAccountMeta(accounts.metadata),
      getAccountMeta(accounts.masterEdition),
      getAccountMeta(accounts.payer),
      getAccountMeta(accounts.logWrapper),
      getAccountMeta(accounts.compressionProgram),
      getAccountMeta(accounts.tokenProgram),
      getAccountMeta(accounts.tokenMetadataProgram),
      getAccountMeta(accounts.systemProgram)
    ],
    programAddress,
    data: getCompressInstructionDataEncoder().encode({})
  };
  return instruction;
}
function parseCompressInstruction(instruction) {
  if (instruction.accounts.length < 14) {
    throw new Error("Not enough accounts");
  }
  let accountIndex = 0;
  const getNextAccount = () => {
    const accountMeta = instruction.accounts[accountIndex];
    accountIndex += 1;
    return accountMeta;
  };
  return {
    programAddress: instruction.programAddress,
    accounts: {
      treeAuthority: getNextAccount(),
      leafOwner: getNextAccount(),
      leafDelegate: getNextAccount(),
      merkleTree: getNextAccount(),
      tokenAccount: getNextAccount(),
      mint: getNextAccount(),
      metadata: getNextAccount(),
      masterEdition: getNextAccount(),
      payer: getNextAccount(),
      logWrapper: getNextAccount(),
      compressionProgram: getNextAccount(),
      tokenProgram: getNextAccount(),
      tokenMetadataProgram: getNextAccount(),
      systemProgram: getNextAccount()
    },
    data: getCompressInstructionDataDecoder().decode(instruction.data)
  };
}
function getCreateTreeInstructionDataEncoder() {
  return web3_js.transformEncoder(
    web3_js.getStructEncoder([
      ["discriminator", web3_js.fixEncoderSize(web3_js.getBytesEncoder(), 8)],
      ["maxDepth", web3_js.getU32Encoder()],
      ["maxBufferSize", web3_js.getU32Encoder()],
      ["public", web3_js.getOptionEncoder(web3_js.getBooleanEncoder())]
    ]),
    (value) => ({
      ...value,
      discriminator: new Uint8Array([165, 83, 136, 142, 89, 202, 47, 220])
    })
  );
}
function getCreateTreeInstructionDataDecoder() {
  return web3_js.getStructDecoder([
    ["discriminator", web3_js.fixDecoderSize(web3_js.getBytesDecoder(), 8)],
    ["maxDepth", web3_js.getU32Decoder()],
    ["maxBufferSize", web3_js.getU32Decoder()],
    ["public", web3_js.getOptionDecoder(web3_js.getBooleanDecoder())]
  ]);
}
function getCreateTreeInstructionDataCodec() {
  return web3_js.combineCodec(
    getCreateTreeInstructionDataEncoder(),
    getCreateTreeInstructionDataDecoder()
  );
}
function getCreateTreeInstruction(input) {
  const programAddress = BUBBLEGUM_PROGRAM_ADDRESS;
  const originalAccounts = {
    treeAuthority: { value: input.treeAuthority ?? null, isWritable: true },
    merkleTree: { value: input.merkleTree ?? null, isWritable: true },
    payer: { value: input.payer ?? null, isWritable: true },
    treeCreator: { value: input.treeCreator ?? null, isWritable: false },
    logWrapper: { value: input.logWrapper ?? null, isWritable: false },
    compressionProgram: {
      value: input.compressionProgram ?? null,
      isWritable: false
    },
    systemProgram: { value: input.systemProgram ?? null, isWritable: false }
  };
  const accounts = originalAccounts;
  const args = { ...input };
  if (!accounts.systemProgram.value) {
    accounts.systemProgram.value = "11111111111111111111111111111111";
  }
  const getAccountMeta = getAccountMetaFactory2(programAddress);
  const instruction = {
    accounts: [
      getAccountMeta(accounts.treeAuthority),
      getAccountMeta(accounts.merkleTree),
      getAccountMeta(accounts.payer),
      getAccountMeta(accounts.treeCreator),
      getAccountMeta(accounts.logWrapper),
      getAccountMeta(accounts.compressionProgram),
      getAccountMeta(accounts.systemProgram)
    ],
    programAddress,
    data: getCreateTreeInstructionDataEncoder().encode(
      args
    )
  };
  return instruction;
}
function parseCreateTreeInstruction(instruction) {
  if (instruction.accounts.length < 7) {
    throw new Error("Not enough accounts");
  }
  let accountIndex = 0;
  const getNextAccount = () => {
    const accountMeta = instruction.accounts[accountIndex];
    accountIndex += 1;
    return accountMeta;
  };
  return {
    programAddress: instruction.programAddress,
    accounts: {
      treeAuthority: getNextAccount(),
      merkleTree: getNextAccount(),
      payer: getNextAccount(),
      treeCreator: getNextAccount(),
      logWrapper: getNextAccount(),
      compressionProgram: getNextAccount(),
      systemProgram: getNextAccount()
    },
    data: getCreateTreeInstructionDataDecoder().decode(instruction.data)
  };
}
function getDecompressV1InstructionDataEncoder() {
  return web3_js.transformEncoder(
    web3_js.getStructEncoder([
      ["discriminator", web3_js.fixEncoderSize(web3_js.getBytesEncoder(), 8)],
      ["metadata", getMetadataArgsEncoder()]
    ]),
    (value) => ({
      ...value,
      discriminator: new Uint8Array([54, 85, 76, 70, 228, 250, 164, 81])
    })
  );
}
function getDecompressV1InstructionDataDecoder() {
  return web3_js.getStructDecoder([
    ["discriminator", web3_js.fixDecoderSize(web3_js.getBytesDecoder(), 8)],
    ["metadata", getMetadataArgsDecoder()]
  ]);
}
function getDecompressV1InstructionDataCodec() {
  return web3_js.combineCodec(
    getDecompressV1InstructionDataEncoder(),
    getDecompressV1InstructionDataDecoder()
  );
}
function getDecompressV1Instruction(input) {
  const programAddress = BUBBLEGUM_PROGRAM_ADDRESS;
  const originalAccounts = {
    voucher: { value: input.voucher ?? null, isWritable: true },
    leafOwner: { value: input.leafOwner ?? null, isWritable: true },
    tokenAccount: { value: input.tokenAccount ?? null, isWritable: true },
    mint: { value: input.mint ?? null, isWritable: true },
    mintAuthority: { value: input.mintAuthority ?? null, isWritable: true },
    metadata: { value: input.metadata ?? null, isWritable: true },
    masterEdition: { value: input.masterEdition ?? null, isWritable: true },
    systemProgram: { value: input.systemProgram ?? null, isWritable: false },
    sysvarRent: { value: input.sysvarRent ?? null, isWritable: false },
    tokenMetadataProgram: {
      value: input.tokenMetadataProgram ?? null,
      isWritable: false
    },
    tokenProgram: { value: input.tokenProgram ?? null, isWritable: false },
    associatedTokenProgram: {
      value: input.associatedTokenProgram ?? null,
      isWritable: false
    },
    logWrapper: { value: input.logWrapper ?? null, isWritable: false }
  };
  const accounts = originalAccounts;
  const args = { ...input, metadata: input.metadataArg };
  if (!accounts.systemProgram.value) {
    accounts.systemProgram.value = "11111111111111111111111111111111";
  }
  if (!accounts.sysvarRent.value) {
    accounts.sysvarRent.value = "SysvarRent111111111111111111111111111111111";
  }
  if (!accounts.tokenMetadataProgram.value) {
    accounts.tokenMetadataProgram.value = "metaqbxxUerdq28cj1RbAWkYQm3ybzjb6a8bt518x1s";
  }
  if (!accounts.tokenProgram.value) {
    accounts.tokenProgram.value = "TokenkegQfeZyiNwAJbNbGKPFXCWuBvf9Ss623VQ5DA";
  }
  const getAccountMeta = getAccountMetaFactory2(programAddress);
  const instruction = {
    accounts: [
      getAccountMeta(accounts.voucher),
      getAccountMeta(accounts.leafOwner),
      getAccountMeta(accounts.tokenAccount),
      getAccountMeta(accounts.mint),
      getAccountMeta(accounts.mintAuthority),
      getAccountMeta(accounts.metadata),
      getAccountMeta(accounts.masterEdition),
      getAccountMeta(accounts.systemProgram),
      getAccountMeta(accounts.sysvarRent),
      getAccountMeta(accounts.tokenMetadataProgram),
      getAccountMeta(accounts.tokenProgram),
      getAccountMeta(accounts.associatedTokenProgram),
      getAccountMeta(accounts.logWrapper)
    ],
    programAddress,
    data: getDecompressV1InstructionDataEncoder().encode(
      args
    )
  };
  return instruction;
}
function parseDecompressV1Instruction(instruction) {
  if (instruction.accounts.length < 13) {
    throw new Error("Not enough accounts");
  }
  let accountIndex = 0;
  const getNextAccount = () => {
    const accountMeta = instruction.accounts[accountIndex];
    accountIndex += 1;
    return accountMeta;
  };
  return {
    programAddress: instruction.programAddress,
    accounts: {
      voucher: getNextAccount(),
      leafOwner: getNextAccount(),
      tokenAccount: getNextAccount(),
      mint: getNextAccount(),
      mintAuthority: getNextAccount(),
      metadata: getNextAccount(),
      masterEdition: getNextAccount(),
      systemProgram: getNextAccount(),
      sysvarRent: getNextAccount(),
      tokenMetadataProgram: getNextAccount(),
      tokenProgram: getNextAccount(),
      associatedTokenProgram: getNextAccount(),
      logWrapper: getNextAccount()
    },
    data: getDecompressV1InstructionDataDecoder().decode(instruction.data)
  };
}
function getDelegateInstructionDataEncoder() {
  return web3_js.transformEncoder(
    web3_js.getStructEncoder([
      ["discriminator", web3_js.fixEncoderSize(web3_js.getBytesEncoder(), 8)],
      ["root", web3_js.fixEncoderSize(web3_js.getBytesEncoder(), 32)],
      ["dataHash", web3_js.fixEncoderSize(web3_js.getBytesEncoder(), 32)],
      ["creatorHash", web3_js.fixEncoderSize(web3_js.getBytesEncoder(), 32)],
      ["nonce", web3_js.getU64Encoder()],
      ["index", web3_js.getU32Encoder()]
    ]),
    (value) => ({
      ...value,
      discriminator: new Uint8Array([90, 147, 75, 178, 85, 88, 4, 137])
    })
  );
}
function getDelegateInstructionDataDecoder() {
  return web3_js.getStructDecoder([
    ["discriminator", web3_js.fixDecoderSize(web3_js.getBytesDecoder(), 8)],
    ["root", web3_js.fixDecoderSize(web3_js.getBytesDecoder(), 32)],
    ["dataHash", web3_js.fixDecoderSize(web3_js.getBytesDecoder(), 32)],
    ["creatorHash", web3_js.fixDecoderSize(web3_js.getBytesDecoder(), 32)],
    ["nonce", web3_js.getU64Decoder()],
    ["index", web3_js.getU32Decoder()]
  ]);
}
function getDelegateInstructionDataCodec() {
  return web3_js.combineCodec(
    getDelegateInstructionDataEncoder(),
    getDelegateInstructionDataDecoder()
  );
}
function getDelegateInstruction(input) {
  const programAddress = BUBBLEGUM_PROGRAM_ADDRESS;
  const originalAccounts = {
    treeAuthority: { value: input.treeAuthority ?? null, isWritable: false },
    leafOwner: { value: input.leafOwner ?? null, isWritable: false },
    previousLeafDelegate: {
      value: input.previousLeafDelegate ?? null,
      isWritable: false
    },
    newLeafDelegate: {
      value: input.newLeafDelegate ?? null,
      isWritable: false
    },
    merkleTree: { value: input.merkleTree ?? null, isWritable: true },
    logWrapper: { value: input.logWrapper ?? null, isWritable: false },
    compressionProgram: {
      value: input.compressionProgram ?? null,
      isWritable: false
    },
    systemProgram: { value: input.systemProgram ?? null, isWritable: false }
  };
  const accounts = originalAccounts;
  const args = { ...input };
  if (!accounts.systemProgram.value) {
    accounts.systemProgram.value = "11111111111111111111111111111111";
  }
  const getAccountMeta = getAccountMetaFactory2(programAddress);
  const instruction = {
    accounts: [
      getAccountMeta(accounts.treeAuthority),
      getAccountMeta(accounts.leafOwner),
      getAccountMeta(accounts.previousLeafDelegate),
      getAccountMeta(accounts.newLeafDelegate),
      getAccountMeta(accounts.merkleTree),
      getAccountMeta(accounts.logWrapper),
      getAccountMeta(accounts.compressionProgram),
      getAccountMeta(accounts.systemProgram)
    ],
    programAddress,
    data: getDelegateInstructionDataEncoder().encode(
      args
    )
  };
  return instruction;
}
function parseDelegateInstruction(instruction) {
  if (instruction.accounts.length < 8) {
    throw new Error("Not enough accounts");
  }
  let accountIndex = 0;
  const getNextAccount = () => {
    const accountMeta = instruction.accounts[accountIndex];
    accountIndex += 1;
    return accountMeta;
  };
  return {
    programAddress: instruction.programAddress,
    accounts: {
      treeAuthority: getNextAccount(),
      leafOwner: getNextAccount(),
      previousLeafDelegate: getNextAccount(),
      newLeafDelegate: getNextAccount(),
      merkleTree: getNextAccount(),
      logWrapper: getNextAccount(),
      compressionProgram: getNextAccount(),
      systemProgram: getNextAccount()
    },
    data: getDelegateInstructionDataDecoder().decode(instruction.data)
  };
}
function getMintToCollectionV1InstructionDataEncoder() {
  return web3_js.transformEncoder(
    web3_js.getStructEncoder([
      ["discriminator", web3_js.fixEncoderSize(web3_js.getBytesEncoder(), 8)],
      ["metadataArgs", getMetadataArgsEncoder()]
    ]),
    (value) => ({
      ...value,
      discriminator: new Uint8Array([153, 18, 178, 47, 197, 158, 86, 15])
    })
  );
}
function getMintToCollectionV1InstructionDataDecoder() {
  return web3_js.getStructDecoder([
    ["discriminator", web3_js.fixDecoderSize(web3_js.getBytesDecoder(), 8)],
    ["metadataArgs", getMetadataArgsDecoder()]
  ]);
}
function getMintToCollectionV1InstructionDataCodec() {
  return web3_js.combineCodec(
    getMintToCollectionV1InstructionDataEncoder(),
    getMintToCollectionV1InstructionDataDecoder()
  );
}
function getMintToCollectionV1Instruction(input) {
  const programAddress = BUBBLEGUM_PROGRAM_ADDRESS;
  const originalAccounts = {
    treeAuthority: { value: input.treeAuthority ?? null, isWritable: true },
    leafOwner: { value: input.leafOwner ?? null, isWritable: false },
    leafDelegate: { value: input.leafDelegate ?? null, isWritable: false },
    merkleTree: { value: input.merkleTree ?? null, isWritable: true },
    payer: { value: input.payer ?? null, isWritable: false },
    treeDelegate: { value: input.treeDelegate ?? null, isWritable: false },
    collectionAuthority: {
      value: input.collectionAuthority ?? null,
      isWritable: false
    },
    collectionAuthorityRecordPda: {
      value: input.collectionAuthorityRecordPda ?? null,
      isWritable: false
    },
    collectionMint: { value: input.collectionMint ?? null, isWritable: false },
    collectionMetadata: {
      value: input.collectionMetadata ?? null,
      isWritable: true
    },
    editionAccount: { value: input.editionAccount ?? null, isWritable: false },
    bubblegumSigner: {
      value: input.bubblegumSigner ?? null,
      isWritable: false
    },
    logWrapper: { value: input.logWrapper ?? null, isWritable: false },
    compressionProgram: {
      value: input.compressionProgram ?? null,
      isWritable: false
    },
    tokenMetadataProgram: {
      value: input.tokenMetadataProgram ?? null,
      isWritable: false
    },
    systemProgram: { value: input.systemProgram ?? null, isWritable: false }
  };
  const accounts = originalAccounts;
  const args = { ...input };
  if (!accounts.tokenMetadataProgram.value) {
    accounts.tokenMetadataProgram.value = "metaqbxxUerdq28cj1RbAWkYQm3ybzjb6a8bt518x1s";
  }
  if (!accounts.systemProgram.value) {
    accounts.systemProgram.value = "11111111111111111111111111111111";
  }
  const getAccountMeta = getAccountMetaFactory2(programAddress);
  const instruction = {
    accounts: [
      getAccountMeta(accounts.treeAuthority),
      getAccountMeta(accounts.leafOwner),
      getAccountMeta(accounts.leafDelegate),
      getAccountMeta(accounts.merkleTree),
      getAccountMeta(accounts.payer),
      getAccountMeta(accounts.treeDelegate),
      getAccountMeta(accounts.collectionAuthority),
      getAccountMeta(accounts.collectionAuthorityRecordPda),
      getAccountMeta(accounts.collectionMint),
      getAccountMeta(accounts.collectionMetadata),
      getAccountMeta(accounts.editionAccount),
      getAccountMeta(accounts.bubblegumSigner),
      getAccountMeta(accounts.logWrapper),
      getAccountMeta(accounts.compressionProgram),
      getAccountMeta(accounts.tokenMetadataProgram),
      getAccountMeta(accounts.systemProgram)
    ],
    programAddress,
    data: getMintToCollectionV1InstructionDataEncoder().encode(
      args
    )
  };
  return instruction;
}
function parseMintToCollectionV1Instruction(instruction) {
  if (instruction.accounts.length < 16) {
    throw new Error("Not enough accounts");
  }
  let accountIndex = 0;
  const getNextAccount = () => {
    const accountMeta = instruction.accounts[accountIndex];
    accountIndex += 1;
    return accountMeta;
  };
  return {
    programAddress: instruction.programAddress,
    accounts: {
      treeAuthority: getNextAccount(),
      leafOwner: getNextAccount(),
      leafDelegate: getNextAccount(),
      merkleTree: getNextAccount(),
      payer: getNextAccount(),
      treeDelegate: getNextAccount(),
      collectionAuthority: getNextAccount(),
      collectionAuthorityRecordPda: getNextAccount(),
      collectionMint: getNextAccount(),
      collectionMetadata: getNextAccount(),
      editionAccount: getNextAccount(),
      bubblegumSigner: getNextAccount(),
      logWrapper: getNextAccount(),
      compressionProgram: getNextAccount(),
      tokenMetadataProgram: getNextAccount(),
      systemProgram: getNextAccount()
    },
    data: getMintToCollectionV1InstructionDataDecoder().decode(
      instruction.data
    )
  };
}
function getMintV1InstructionDataEncoder() {
  return web3_js.transformEncoder(
    web3_js.getStructEncoder([
      ["discriminator", web3_js.fixEncoderSize(web3_js.getBytesEncoder(), 8)],
      ["message", getMetadataArgsEncoder()]
    ]),
    (value) => ({
      ...value,
      discriminator: new Uint8Array([145, 98, 192, 118, 184, 147, 118, 104])
    })
  );
}
function getMintV1InstructionDataDecoder() {
  return web3_js.getStructDecoder([
    ["discriminator", web3_js.fixDecoderSize(web3_js.getBytesDecoder(), 8)],
    ["message", getMetadataArgsDecoder()]
  ]);
}
function getMintV1InstructionDataCodec() {
  return web3_js.combineCodec(
    getMintV1InstructionDataEncoder(),
    getMintV1InstructionDataDecoder()
  );
}
function getMintV1Instruction(input) {
  const programAddress = BUBBLEGUM_PROGRAM_ADDRESS;
  const originalAccounts = {
    treeAuthority: { value: input.treeAuthority ?? null, isWritable: true },
    leafOwner: { value: input.leafOwner ?? null, isWritable: false },
    leafDelegate: { value: input.leafDelegate ?? null, isWritable: false },
    merkleTree: { value: input.merkleTree ?? null, isWritable: true },
    payer: { value: input.payer ?? null, isWritable: false },
    treeDelegate: { value: input.treeDelegate ?? null, isWritable: false },
    logWrapper: { value: input.logWrapper ?? null, isWritable: false },
    compressionProgram: {
      value: input.compressionProgram ?? null,
      isWritable: false
    },
    systemProgram: { value: input.systemProgram ?? null, isWritable: false }
  };
  const accounts = originalAccounts;
  const args = { ...input };
  if (!accounts.systemProgram.value) {
    accounts.systemProgram.value = "11111111111111111111111111111111";
  }
  const getAccountMeta = getAccountMetaFactory2(programAddress);
  const instruction = {
    accounts: [
      getAccountMeta(accounts.treeAuthority),
      getAccountMeta(accounts.leafOwner),
      getAccountMeta(accounts.leafDelegate),
      getAccountMeta(accounts.merkleTree),
      getAccountMeta(accounts.payer),
      getAccountMeta(accounts.treeDelegate),
      getAccountMeta(accounts.logWrapper),
      getAccountMeta(accounts.compressionProgram),
      getAccountMeta(accounts.systemProgram)
    ],
    programAddress,
    data: getMintV1InstructionDataEncoder().encode(
      args
    )
  };
  return instruction;
}
function parseMintV1Instruction(instruction) {
  if (instruction.accounts.length < 9) {
    throw new Error("Not enough accounts");
  }
  let accountIndex = 0;
  const getNextAccount = () => {
    const accountMeta = instruction.accounts[accountIndex];
    accountIndex += 1;
    return accountMeta;
  };
  return {
    programAddress: instruction.programAddress,
    accounts: {
      treeAuthority: getNextAccount(),
      leafOwner: getNextAccount(),
      leafDelegate: getNextAccount(),
      merkleTree: getNextAccount(),
      payer: getNextAccount(),
      treeDelegate: getNextAccount(),
      logWrapper: getNextAccount(),
      compressionProgram: getNextAccount(),
      systemProgram: getNextAccount()
    },
    data: getMintV1InstructionDataDecoder().decode(instruction.data)
  };
}
function getRedeemInstructionDataEncoder() {
  return web3_js.transformEncoder(
    web3_js.getStructEncoder([
      ["discriminator", web3_js.fixEncoderSize(web3_js.getBytesEncoder(), 8)],
      ["root", web3_js.fixEncoderSize(web3_js.getBytesEncoder(), 32)],
      ["dataHash", web3_js.fixEncoderSize(web3_js.getBytesEncoder(), 32)],
      ["creatorHash", web3_js.fixEncoderSize(web3_js.getBytesEncoder(), 32)],
      ["nonce", web3_js.getU64Encoder()],
      ["index", web3_js.getU32Encoder()]
    ]),
    (value) => ({
      ...value,
      discriminator: new Uint8Array([184, 12, 86, 149, 70, 196, 97, 225])
    })
  );
}
function getRedeemInstructionDataDecoder() {
  return web3_js.getStructDecoder([
    ["discriminator", web3_js.fixDecoderSize(web3_js.getBytesDecoder(), 8)],
    ["root", web3_js.fixDecoderSize(web3_js.getBytesDecoder(), 32)],
    ["dataHash", web3_js.fixDecoderSize(web3_js.getBytesDecoder(), 32)],
    ["creatorHash", web3_js.fixDecoderSize(web3_js.getBytesDecoder(), 32)],
    ["nonce", web3_js.getU64Decoder()],
    ["index", web3_js.getU32Decoder()]
  ]);
}
function getRedeemInstructionDataCodec() {
  return web3_js.combineCodec(
    getRedeemInstructionDataEncoder(),
    getRedeemInstructionDataDecoder()
  );
}
function getRedeemInstruction(input) {
  const programAddress = BUBBLEGUM_PROGRAM_ADDRESS;
  const originalAccounts = {
    treeAuthority: { value: input.treeAuthority ?? null, isWritable: false },
    leafOwner: { value: input.leafOwner ?? null, isWritable: true },
    leafDelegate: { value: input.leafDelegate ?? null, isWritable: false },
    merkleTree: { value: input.merkleTree ?? null, isWritable: true },
    voucher: { value: input.voucher ?? null, isWritable: true },
    logWrapper: { value: input.logWrapper ?? null, isWritable: false },
    compressionProgram: {
      value: input.compressionProgram ?? null,
      isWritable: false
    },
    systemProgram: { value: input.systemProgram ?? null, isWritable: false }
  };
  const accounts = originalAccounts;
  const args = { ...input };
  if (!accounts.systemProgram.value) {
    accounts.systemProgram.value = "11111111111111111111111111111111";
  }
  const getAccountMeta = getAccountMetaFactory2(programAddress);
  const instruction = {
    accounts: [
      getAccountMeta(accounts.treeAuthority),
      getAccountMeta(accounts.leafOwner),
      getAccountMeta(accounts.leafDelegate),
      getAccountMeta(accounts.merkleTree),
      getAccountMeta(accounts.voucher),
      getAccountMeta(accounts.logWrapper),
      getAccountMeta(accounts.compressionProgram),
      getAccountMeta(accounts.systemProgram)
    ],
    programAddress,
    data: getRedeemInstructionDataEncoder().encode(
      args
    )
  };
  return instruction;
}
function parseRedeemInstruction(instruction) {
  if (instruction.accounts.length < 8) {
    throw new Error("Not enough accounts");
  }
  let accountIndex = 0;
  const getNextAccount = () => {
    const accountMeta = instruction.accounts[accountIndex];
    accountIndex += 1;
    return accountMeta;
  };
  return {
    programAddress: instruction.programAddress,
    accounts: {
      treeAuthority: getNextAccount(),
      leafOwner: getNextAccount(),
      leafDelegate: getNextAccount(),
      merkleTree: getNextAccount(),
      voucher: getNextAccount(),
      logWrapper: getNextAccount(),
      compressionProgram: getNextAccount(),
      systemProgram: getNextAccount()
    },
    data: getRedeemInstructionDataDecoder().decode(instruction.data)
  };
}
function getSetAndVerifyCollectionInstructionDataEncoder() {
  return web3_js.transformEncoder(
    web3_js.getStructEncoder([
      ["discriminator", web3_js.fixEncoderSize(web3_js.getBytesEncoder(), 8)],
      ["root", web3_js.fixEncoderSize(web3_js.getBytesEncoder(), 32)],
      ["dataHash", web3_js.fixEncoderSize(web3_js.getBytesEncoder(), 32)],
      ["creatorHash", web3_js.fixEncoderSize(web3_js.getBytesEncoder(), 32)],
      ["nonce", web3_js.getU64Encoder()],
      ["index", web3_js.getU32Encoder()],
      ["message", getMetadataArgsEncoder()],
      ["collection", web3_js.getAddressEncoder()]
    ]),
    (value) => ({
      ...value,
      discriminator: new Uint8Array([235, 242, 121, 216, 158, 234, 180, 234])
    })
  );
}
function getSetAndVerifyCollectionInstructionDataDecoder() {
  return web3_js.getStructDecoder([
    ["discriminator", web3_js.fixDecoderSize(web3_js.getBytesDecoder(), 8)],
    ["root", web3_js.fixDecoderSize(web3_js.getBytesDecoder(), 32)],
    ["dataHash", web3_js.fixDecoderSize(web3_js.getBytesDecoder(), 32)],
    ["creatorHash", web3_js.fixDecoderSize(web3_js.getBytesDecoder(), 32)],
    ["nonce", web3_js.getU64Decoder()],
    ["index", web3_js.getU32Decoder()],
    ["message", getMetadataArgsDecoder()],
    ["collection", web3_js.getAddressDecoder()]
  ]);
}
function getSetAndVerifyCollectionInstructionDataCodec() {
  return web3_js.combineCodec(
    getSetAndVerifyCollectionInstructionDataEncoder(),
    getSetAndVerifyCollectionInstructionDataDecoder()
  );
}
function getSetAndVerifyCollectionInstruction(input) {
  const programAddress = BUBBLEGUM_PROGRAM_ADDRESS;
  const originalAccounts = {
    treeAuthority: { value: input.treeAuthority ?? null, isWritable: false },
    leafOwner: { value: input.leafOwner ?? null, isWritable: false },
    leafDelegate: { value: input.leafDelegate ?? null, isWritable: false },
    merkleTree: { value: input.merkleTree ?? null, isWritable: true },
    payer: { value: input.payer ?? null, isWritable: false },
    treeDelegate: { value: input.treeDelegate ?? null, isWritable: false },
    collectionAuthority: {
      value: input.collectionAuthority ?? null,
      isWritable: false
    },
    collectionAuthorityRecordPda: {
      value: input.collectionAuthorityRecordPda ?? null,
      isWritable: false
    },
    collectionMint: { value: input.collectionMint ?? null, isWritable: false },
    collectionMetadata: {
      value: input.collectionMetadata ?? null,
      isWritable: true
    },
    editionAccount: { value: input.editionAccount ?? null, isWritable: false },
    bubblegumSigner: {
      value: input.bubblegumSigner ?? null,
      isWritable: false
    },
    logWrapper: { value: input.logWrapper ?? null, isWritable: false },
    compressionProgram: {
      value: input.compressionProgram ?? null,
      isWritable: false
    },
    tokenMetadataProgram: {
      value: input.tokenMetadataProgram ?? null,
      isWritable: false
    },
    systemProgram: { value: input.systemProgram ?? null, isWritable: false }
  };
  const accounts = originalAccounts;
  const args = { ...input };
  if (!accounts.tokenMetadataProgram.value) {
    accounts.tokenMetadataProgram.value = "metaqbxxUerdq28cj1RbAWkYQm3ybzjb6a8bt518x1s";
  }
  if (!accounts.systemProgram.value) {
    accounts.systemProgram.value = "11111111111111111111111111111111";
  }
  const getAccountMeta = getAccountMetaFactory2(programAddress);
  const instruction = {
    accounts: [
      getAccountMeta(accounts.treeAuthority),
      getAccountMeta(accounts.leafOwner),
      getAccountMeta(accounts.leafDelegate),
      getAccountMeta(accounts.merkleTree),
      getAccountMeta(accounts.payer),
      getAccountMeta(accounts.treeDelegate),
      getAccountMeta(accounts.collectionAuthority),
      getAccountMeta(accounts.collectionAuthorityRecordPda),
      getAccountMeta(accounts.collectionMint),
      getAccountMeta(accounts.collectionMetadata),
      getAccountMeta(accounts.editionAccount),
      getAccountMeta(accounts.bubblegumSigner),
      getAccountMeta(accounts.logWrapper),
      getAccountMeta(accounts.compressionProgram),
      getAccountMeta(accounts.tokenMetadataProgram),
      getAccountMeta(accounts.systemProgram)
    ],
    programAddress,
    data: getSetAndVerifyCollectionInstructionDataEncoder().encode(
      args
    )
  };
  return instruction;
}
function parseSetAndVerifyCollectionInstruction(instruction) {
  if (instruction.accounts.length < 16) {
    throw new Error("Not enough accounts");
  }
  let accountIndex = 0;
  const getNextAccount = () => {
    const accountMeta = instruction.accounts[accountIndex];
    accountIndex += 1;
    return accountMeta;
  };
  return {
    programAddress: instruction.programAddress,
    accounts: {
      treeAuthority: getNextAccount(),
      leafOwner: getNextAccount(),
      leafDelegate: getNextAccount(),
      merkleTree: getNextAccount(),
      payer: getNextAccount(),
      treeDelegate: getNextAccount(),
      collectionAuthority: getNextAccount(),
      collectionAuthorityRecordPda: getNextAccount(),
      collectionMint: getNextAccount(),
      collectionMetadata: getNextAccount(),
      editionAccount: getNextAccount(),
      bubblegumSigner: getNextAccount(),
      logWrapper: getNextAccount(),
      compressionProgram: getNextAccount(),
      tokenMetadataProgram: getNextAccount(),
      systemProgram: getNextAccount()
    },
    data: getSetAndVerifyCollectionInstructionDataDecoder().decode(
      instruction.data
    )
  };
}
function getSetDecompressableStateInstructionDataEncoder() {
  return web3_js.transformEncoder(
    web3_js.getStructEncoder([
      ["discriminator", web3_js.fixEncoderSize(web3_js.getBytesEncoder(), 8)],
      ["decompressableState", getDecompressibleStateEncoder()]
    ]),
    (value) => ({
      ...value,
      discriminator: new Uint8Array([18, 135, 238, 168, 246, 195, 61, 115])
    })
  );
}
function getSetDecompressableStateInstructionDataDecoder() {
  return web3_js.getStructDecoder([
    ["discriminator", web3_js.fixDecoderSize(web3_js.getBytesDecoder(), 8)],
    ["decompressableState", getDecompressibleStateDecoder()]
  ]);
}
function getSetDecompressableStateInstructionDataCodec() {
  return web3_js.combineCodec(
    getSetDecompressableStateInstructionDataEncoder(),
    getSetDecompressableStateInstructionDataDecoder()
  );
}
function getSetDecompressableStateInstruction(input) {
  const programAddress = BUBBLEGUM_PROGRAM_ADDRESS;
  const originalAccounts = {
    treeAuthority: { value: input.treeAuthority ?? null, isWritable: true },
    treeCreator: { value: input.treeCreator ?? null, isWritable: false }
  };
  const accounts = originalAccounts;
  const args = { ...input };
  const getAccountMeta = getAccountMetaFactory2(programAddress);
  const instruction = {
    accounts: [
      getAccountMeta(accounts.treeAuthority),
      getAccountMeta(accounts.treeCreator)
    ],
    programAddress,
    data: getSetDecompressableStateInstructionDataEncoder().encode(
      args
    )
  };
  return instruction;
}
function parseSetDecompressableStateInstruction(instruction) {
  if (instruction.accounts.length < 2) {
    throw new Error("Not enough accounts");
  }
  let accountIndex = 0;
  const getNextAccount = () => {
    const accountMeta = instruction.accounts[accountIndex];
    accountIndex += 1;
    return accountMeta;
  };
  return {
    programAddress: instruction.programAddress,
    accounts: {
      treeAuthority: getNextAccount(),
      treeCreator: getNextAccount()
    },
    data: getSetDecompressableStateInstructionDataDecoder().decode(
      instruction.data
    )
  };
}
function getSetDecompressibleStateInstructionDataEncoder() {
  return web3_js.transformEncoder(
    web3_js.getStructEncoder([
      ["discriminator", web3_js.fixEncoderSize(web3_js.getBytesEncoder(), 8)],
      ["decompressableState", getDecompressibleStateEncoder()]
    ]),
    (value) => ({
      ...value,
      discriminator: new Uint8Array([82, 104, 152, 6, 149, 111, 100, 13])
    })
  );
}
function getSetDecompressibleStateInstructionDataDecoder() {
  return web3_js.getStructDecoder([
    ["discriminator", web3_js.fixDecoderSize(web3_js.getBytesDecoder(), 8)],
    ["decompressableState", getDecompressibleStateDecoder()]
  ]);
}
function getSetDecompressibleStateInstructionDataCodec() {
  return web3_js.combineCodec(
    getSetDecompressibleStateInstructionDataEncoder(),
    getSetDecompressibleStateInstructionDataDecoder()
  );
}
function getSetDecompressibleStateInstruction(input) {
  const programAddress = BUBBLEGUM_PROGRAM_ADDRESS;
  const originalAccounts = {
    treeAuthority: { value: input.treeAuthority ?? null, isWritable: true },
    treeCreator: { value: input.treeCreator ?? null, isWritable: false }
  };
  const accounts = originalAccounts;
  const args = { ...input };
  const getAccountMeta = getAccountMetaFactory2(programAddress);
  const instruction = {
    accounts: [
      getAccountMeta(accounts.treeAuthority),
      getAccountMeta(accounts.treeCreator)
    ],
    programAddress,
    data: getSetDecompressibleStateInstructionDataEncoder().encode(
      args
    )
  };
  return instruction;
}
function parseSetDecompressibleStateInstruction(instruction) {
  if (instruction.accounts.length < 2) {
    throw new Error("Not enough accounts");
  }
  let accountIndex = 0;
  const getNextAccount = () => {
    const accountMeta = instruction.accounts[accountIndex];
    accountIndex += 1;
    return accountMeta;
  };
  return {
    programAddress: instruction.programAddress,
    accounts: {
      treeAuthority: getNextAccount(),
      treeCreator: getNextAccount()
    },
    data: getSetDecompressibleStateInstructionDataDecoder().decode(
      instruction.data
    )
  };
}
function getSetTreeDelegateInstructionDataEncoder() {
  return web3_js.transformEncoder(
    web3_js.getStructEncoder([["discriminator", web3_js.fixEncoderSize(web3_js.getBytesEncoder(), 8)]]),
    (value) => ({
      ...value,
      discriminator: new Uint8Array([253, 118, 66, 37, 190, 49, 154, 102])
    })
  );
}
function getSetTreeDelegateInstructionDataDecoder() {
  return web3_js.getStructDecoder([
    ["discriminator", web3_js.fixDecoderSize(web3_js.getBytesDecoder(), 8)]
  ]);
}
function getSetTreeDelegateInstructionDataCodec() {
  return web3_js.combineCodec(
    getSetTreeDelegateInstructionDataEncoder(),
    getSetTreeDelegateInstructionDataDecoder()
  );
}
function getSetTreeDelegateInstruction(input) {
  const programAddress = BUBBLEGUM_PROGRAM_ADDRESS;
  const originalAccounts = {
    treeAuthority: { value: input.treeAuthority ?? null, isWritable: true },
    treeCreator: { value: input.treeCreator ?? null, isWritable: false },
    newTreeDelegate: {
      value: input.newTreeDelegate ?? null,
      isWritable: false
    },
    merkleTree: { value: input.merkleTree ?? null, isWritable: false },
    systemProgram: { value: input.systemProgram ?? null, isWritable: false }
  };
  const accounts = originalAccounts;
  if (!accounts.systemProgram.value) {
    accounts.systemProgram.value = "11111111111111111111111111111111";
  }
  const getAccountMeta = getAccountMetaFactory2(programAddress);
  const instruction = {
    accounts: [
      getAccountMeta(accounts.treeAuthority),
      getAccountMeta(accounts.treeCreator),
      getAccountMeta(accounts.newTreeDelegate),
      getAccountMeta(accounts.merkleTree),
      getAccountMeta(accounts.systemProgram)
    ],
    programAddress,
    data: getSetTreeDelegateInstructionDataEncoder().encode({})
  };
  return instruction;
}
function parseSetTreeDelegateInstruction(instruction) {
  if (instruction.accounts.length < 5) {
    throw new Error("Not enough accounts");
  }
  let accountIndex = 0;
  const getNextAccount = () => {
    const accountMeta = instruction.accounts[accountIndex];
    accountIndex += 1;
    return accountMeta;
  };
  return {
    programAddress: instruction.programAddress,
    accounts: {
      treeAuthority: getNextAccount(),
      treeCreator: getNextAccount(),
      newTreeDelegate: getNextAccount(),
      merkleTree: getNextAccount(),
      systemProgram: getNextAccount()
    },
    data: getSetTreeDelegateInstructionDataDecoder().decode(instruction.data)
  };
}
function getTransferInstructionDataEncoder() {
  return web3_js.transformEncoder(
    web3_js.getStructEncoder([
      ["discriminator", web3_js.fixEncoderSize(web3_js.getBytesEncoder(), 8)],
      ["root", web3_js.fixEncoderSize(web3_js.getBytesEncoder(), 32)],
      ["dataHash", web3_js.fixEncoderSize(web3_js.getBytesEncoder(), 32)],
      ["creatorHash", web3_js.fixEncoderSize(web3_js.getBytesEncoder(), 32)],
      ["nonce", web3_js.getU64Encoder()],
      ["index", web3_js.getU32Encoder()]
    ]),
    (value) => ({
      ...value,
      discriminator: new Uint8Array([163, 52, 200, 231, 140, 3, 69, 186])
    })
  );
}
function getTransferInstructionDataDecoder() {
  return web3_js.getStructDecoder([
    ["discriminator", web3_js.fixDecoderSize(web3_js.getBytesDecoder(), 8)],
    ["root", web3_js.fixDecoderSize(web3_js.getBytesDecoder(), 32)],
    ["dataHash", web3_js.fixDecoderSize(web3_js.getBytesDecoder(), 32)],
    ["creatorHash", web3_js.fixDecoderSize(web3_js.getBytesDecoder(), 32)],
    ["nonce", web3_js.getU64Decoder()],
    ["index", web3_js.getU32Decoder()]
  ]);
}
function getTransferInstructionDataCodec() {
  return web3_js.combineCodec(
    getTransferInstructionDataEncoder(),
    getTransferInstructionDataDecoder()
  );
}
function getTransferInstruction(input) {
  const programAddress = BUBBLEGUM_PROGRAM_ADDRESS;
  const originalAccounts = {
    treeAuthority: { value: input.treeAuthority ?? null, isWritable: false },
    leafOwner: { value: input.leafOwner ?? null, isWritable: false },
    leafDelegate: { value: input.leafDelegate ?? null, isWritable: false },
    newLeafOwner: { value: input.newLeafOwner ?? null, isWritable: false },
    merkleTree: { value: input.merkleTree ?? null, isWritable: true },
    logWrapper: { value: input.logWrapper ?? null, isWritable: false },
    compressionProgram: {
      value: input.compressionProgram ?? null,
      isWritable: false
    },
    systemProgram: { value: input.systemProgram ?? null, isWritable: false }
  };
  const accounts = originalAccounts;
  const args = { ...input };
  if (!accounts.systemProgram.value) {
    accounts.systemProgram.value = "11111111111111111111111111111111";
  }
  const getAccountMeta = getAccountMetaFactory2(programAddress);
  const instruction = {
    accounts: [
      getAccountMeta(accounts.treeAuthority),
      getAccountMeta(accounts.leafOwner),
      getAccountMeta(accounts.leafDelegate),
      getAccountMeta(accounts.newLeafOwner),
      getAccountMeta(accounts.merkleTree),
      getAccountMeta(accounts.logWrapper),
      getAccountMeta(accounts.compressionProgram),
      getAccountMeta(accounts.systemProgram)
    ],
    programAddress,
    data: getTransferInstructionDataEncoder().encode(
      args
    )
  };
  return instruction;
}
function parseTransferInstruction(instruction) {
  if (instruction.accounts.length < 8) {
    throw new Error("Not enough accounts");
  }
  let accountIndex = 0;
  const getNextAccount = () => {
    const accountMeta = instruction.accounts[accountIndex];
    accountIndex += 1;
    return accountMeta;
  };
  return {
    programAddress: instruction.programAddress,
    accounts: {
      treeAuthority: getNextAccount(),
      leafOwner: getNextAccount(),
      leafDelegate: getNextAccount(),
      newLeafOwner: getNextAccount(),
      merkleTree: getNextAccount(),
      logWrapper: getNextAccount(),
      compressionProgram: getNextAccount(),
      systemProgram: getNextAccount()
    },
    data: getTransferInstructionDataDecoder().decode(instruction.data)
  };
}
function getUnverifyCollectionInstructionDataEncoder() {
  return web3_js.transformEncoder(
    web3_js.getStructEncoder([
      ["discriminator", web3_js.fixEncoderSize(web3_js.getBytesEncoder(), 8)],
      ["root", web3_js.fixEncoderSize(web3_js.getBytesEncoder(), 32)],
      ["dataHash", web3_js.fixEncoderSize(web3_js.getBytesEncoder(), 32)],
      ["creatorHash", web3_js.fixEncoderSize(web3_js.getBytesEncoder(), 32)],
      ["nonce", web3_js.getU64Encoder()],
      ["index", web3_js.getU32Encoder()],
      ["message", getMetadataArgsEncoder()]
    ]),
    (value) => ({
      ...value,
      discriminator: new Uint8Array([250, 251, 42, 106, 41, 137, 186, 168])
    })
  );
}
function getUnverifyCollectionInstructionDataDecoder() {
  return web3_js.getStructDecoder([
    ["discriminator", web3_js.fixDecoderSize(web3_js.getBytesDecoder(), 8)],
    ["root", web3_js.fixDecoderSize(web3_js.getBytesDecoder(), 32)],
    ["dataHash", web3_js.fixDecoderSize(web3_js.getBytesDecoder(), 32)],
    ["creatorHash", web3_js.fixDecoderSize(web3_js.getBytesDecoder(), 32)],
    ["nonce", web3_js.getU64Decoder()],
    ["index", web3_js.getU32Decoder()],
    ["message", getMetadataArgsDecoder()]
  ]);
}
function getUnverifyCollectionInstructionDataCodec() {
  return web3_js.combineCodec(
    getUnverifyCollectionInstructionDataEncoder(),
    getUnverifyCollectionInstructionDataDecoder()
  );
}
function getUnverifyCollectionInstruction(input) {
  const programAddress = BUBBLEGUM_PROGRAM_ADDRESS;
  const originalAccounts = {
    treeAuthority: { value: input.treeAuthority ?? null, isWritable: false },
    leafOwner: { value: input.leafOwner ?? null, isWritable: false },
    leafDelegate: { value: input.leafDelegate ?? null, isWritable: false },
    merkleTree: { value: input.merkleTree ?? null, isWritable: true },
    payer: { value: input.payer ?? null, isWritable: false },
    treeDelegate: { value: input.treeDelegate ?? null, isWritable: false },
    collectionAuthority: {
      value: input.collectionAuthority ?? null,
      isWritable: false
    },
    collectionAuthorityRecordPda: {
      value: input.collectionAuthorityRecordPda ?? null,
      isWritable: false
    },
    collectionMint: { value: input.collectionMint ?? null, isWritable: false },
    collectionMetadata: {
      value: input.collectionMetadata ?? null,
      isWritable: true
    },
    editionAccount: { value: input.editionAccount ?? null, isWritable: false },
    bubblegumSigner: {
      value: input.bubblegumSigner ?? null,
      isWritable: false
    },
    logWrapper: { value: input.logWrapper ?? null, isWritable: false },
    compressionProgram: {
      value: input.compressionProgram ?? null,
      isWritable: false
    },
    tokenMetadataProgram: {
      value: input.tokenMetadataProgram ?? null,
      isWritable: false
    },
    systemProgram: { value: input.systemProgram ?? null, isWritable: false }
  };
  const accounts = originalAccounts;
  const args = { ...input };
  if (!accounts.tokenMetadataProgram.value) {
    accounts.tokenMetadataProgram.value = "metaqbxxUerdq28cj1RbAWkYQm3ybzjb6a8bt518x1s";
  }
  if (!accounts.systemProgram.value) {
    accounts.systemProgram.value = "11111111111111111111111111111111";
  }
  const getAccountMeta = getAccountMetaFactory2(programAddress);
  const instruction = {
    accounts: [
      getAccountMeta(accounts.treeAuthority),
      getAccountMeta(accounts.leafOwner),
      getAccountMeta(accounts.leafDelegate),
      getAccountMeta(accounts.merkleTree),
      getAccountMeta(accounts.payer),
      getAccountMeta(accounts.treeDelegate),
      getAccountMeta(accounts.collectionAuthority),
      getAccountMeta(accounts.collectionAuthorityRecordPda),
      getAccountMeta(accounts.collectionMint),
      getAccountMeta(accounts.collectionMetadata),
      getAccountMeta(accounts.editionAccount),
      getAccountMeta(accounts.bubblegumSigner),
      getAccountMeta(accounts.logWrapper),
      getAccountMeta(accounts.compressionProgram),
      getAccountMeta(accounts.tokenMetadataProgram),
      getAccountMeta(accounts.systemProgram)
    ],
    programAddress,
    data: getUnverifyCollectionInstructionDataEncoder().encode(
      args
    )
  };
  return instruction;
}
function parseUnverifyCollectionInstruction(instruction) {
  if (instruction.accounts.length < 16) {
    throw new Error("Not enough accounts");
  }
  let accountIndex = 0;
  const getNextAccount = () => {
    const accountMeta = instruction.accounts[accountIndex];
    accountIndex += 1;
    return accountMeta;
  };
  return {
    programAddress: instruction.programAddress,
    accounts: {
      treeAuthority: getNextAccount(),
      leafOwner: getNextAccount(),
      leafDelegate: getNextAccount(),
      merkleTree: getNextAccount(),
      payer: getNextAccount(),
      treeDelegate: getNextAccount(),
      collectionAuthority: getNextAccount(),
      collectionAuthorityRecordPda: getNextAccount(),
      collectionMint: getNextAccount(),
      collectionMetadata: getNextAccount(),
      editionAccount: getNextAccount(),
      bubblegumSigner: getNextAccount(),
      logWrapper: getNextAccount(),
      compressionProgram: getNextAccount(),
      tokenMetadataProgram: getNextAccount(),
      systemProgram: getNextAccount()
    },
    data: getUnverifyCollectionInstructionDataDecoder().decode(
      instruction.data
    )
  };
}
function getUnverifyCreatorInstructionDataEncoder() {
  return web3_js.transformEncoder(
    web3_js.getStructEncoder([
      ["discriminator", web3_js.fixEncoderSize(web3_js.getBytesEncoder(), 8)],
      ["root", web3_js.fixEncoderSize(web3_js.getBytesEncoder(), 32)],
      ["dataHash", web3_js.fixEncoderSize(web3_js.getBytesEncoder(), 32)],
      ["creatorHash", web3_js.fixEncoderSize(web3_js.getBytesEncoder(), 32)],
      ["nonce", web3_js.getU64Encoder()],
      ["index", web3_js.getU32Encoder()],
      ["message", getMetadataArgsEncoder()]
    ]),
    (value) => ({
      ...value,
      discriminator: new Uint8Array([107, 178, 57, 39, 105, 115, 112, 152])
    })
  );
}
function getUnverifyCreatorInstructionDataDecoder() {
  return web3_js.getStructDecoder([
    ["discriminator", web3_js.fixDecoderSize(web3_js.getBytesDecoder(), 8)],
    ["root", web3_js.fixDecoderSize(web3_js.getBytesDecoder(), 32)],
    ["dataHash", web3_js.fixDecoderSize(web3_js.getBytesDecoder(), 32)],
    ["creatorHash", web3_js.fixDecoderSize(web3_js.getBytesDecoder(), 32)],
    ["nonce", web3_js.getU64Decoder()],
    ["index", web3_js.getU32Decoder()],
    ["message", getMetadataArgsDecoder()]
  ]);
}
function getUnverifyCreatorInstructionDataCodec() {
  return web3_js.combineCodec(
    getUnverifyCreatorInstructionDataEncoder(),
    getUnverifyCreatorInstructionDataDecoder()
  );
}
function getUnverifyCreatorInstruction(input) {
  const programAddress = BUBBLEGUM_PROGRAM_ADDRESS;
  const originalAccounts = {
    treeAuthority: { value: input.treeAuthority ?? null, isWritable: false },
    leafOwner: { value: input.leafOwner ?? null, isWritable: false },
    leafDelegate: { value: input.leafDelegate ?? null, isWritable: false },
    merkleTree: { value: input.merkleTree ?? null, isWritable: true },
    payer: { value: input.payer ?? null, isWritable: false },
    creator: { value: input.creator ?? null, isWritable: false },
    logWrapper: { value: input.logWrapper ?? null, isWritable: false },
    compressionProgram: {
      value: input.compressionProgram ?? null,
      isWritable: false
    },
    systemProgram: { value: input.systemProgram ?? null, isWritable: false }
  };
  const accounts = originalAccounts;
  const args = { ...input };
  if (!accounts.systemProgram.value) {
    accounts.systemProgram.value = "11111111111111111111111111111111";
  }
  const getAccountMeta = getAccountMetaFactory2(programAddress);
  const instruction = {
    accounts: [
      getAccountMeta(accounts.treeAuthority),
      getAccountMeta(accounts.leafOwner),
      getAccountMeta(accounts.leafDelegate),
      getAccountMeta(accounts.merkleTree),
      getAccountMeta(accounts.payer),
      getAccountMeta(accounts.creator),
      getAccountMeta(accounts.logWrapper),
      getAccountMeta(accounts.compressionProgram),
      getAccountMeta(accounts.systemProgram)
    ],
    programAddress,
    data: getUnverifyCreatorInstructionDataEncoder().encode(
      args
    )
  };
  return instruction;
}
function parseUnverifyCreatorInstruction(instruction) {
  if (instruction.accounts.length < 9) {
    throw new Error("Not enough accounts");
  }
  let accountIndex = 0;
  const getNextAccount = () => {
    const accountMeta = instruction.accounts[accountIndex];
    accountIndex += 1;
    return accountMeta;
  };
  return {
    programAddress: instruction.programAddress,
    accounts: {
      treeAuthority: getNextAccount(),
      leafOwner: getNextAccount(),
      leafDelegate: getNextAccount(),
      merkleTree: getNextAccount(),
      payer: getNextAccount(),
      creator: getNextAccount(),
      logWrapper: getNextAccount(),
      compressionProgram: getNextAccount(),
      systemProgram: getNextAccount()
    },
    data: getUnverifyCreatorInstructionDataDecoder().decode(instruction.data)
  };
}
function getUpdateMetadataInstructionDataEncoder() {
  return web3_js.transformEncoder(
    web3_js.getStructEncoder([
      ["discriminator", web3_js.fixEncoderSize(web3_js.getBytesEncoder(), 8)],
      ["root", web3_js.fixEncoderSize(web3_js.getBytesEncoder(), 32)],
      ["nonce", web3_js.getU64Encoder()],
      ["index", web3_js.getU32Encoder()],
      ["currentMetadata", getMetadataArgsEncoder()],
      [
        "name",
        web3_js.getOptionEncoder(
          web3_js.addEncoderSizePrefix(web3_js.getUtf8Encoder(), web3_js.getU32Encoder())
        )
      ],
      [
        "symbol",
        web3_js.getOptionEncoder(
          web3_js.addEncoderSizePrefix(web3_js.getUtf8Encoder(), web3_js.getU32Encoder())
        )
      ],
      [
        "uri",
        web3_js.getOptionEncoder(
          web3_js.addEncoderSizePrefix(web3_js.getUtf8Encoder(), web3_js.getU32Encoder())
        )
      ],
      ["creators", web3_js.getOptionEncoder(web3_js.getArrayEncoder(getCreatorEncoder()))],
      ["sellerFeeBasisPoints", web3_js.getOptionEncoder(web3_js.getU16Encoder())],
      ["primarySaleHappened", web3_js.getOptionEncoder(web3_js.getBooleanEncoder())],
      ["isMutable", web3_js.getOptionEncoder(web3_js.getBooleanEncoder())]
    ]),
    (value) => ({
      ...value,
      discriminator: new Uint8Array([170, 182, 43, 239, 97, 78, 225, 186])
    })
  );
}
function getUpdateMetadataInstructionDataDecoder() {
  return web3_js.getStructDecoder([
    ["discriminator", web3_js.fixDecoderSize(web3_js.getBytesDecoder(), 8)],
    ["root", web3_js.fixDecoderSize(web3_js.getBytesDecoder(), 32)],
    ["nonce", web3_js.getU64Decoder()],
    ["index", web3_js.getU32Decoder()],
    ["currentMetadata", getMetadataArgsDecoder()],
    [
      "name",
      web3_js.getOptionDecoder(web3_js.addDecoderSizePrefix(web3_js.getUtf8Decoder(), web3_js.getU32Decoder()))
    ],
    [
      "symbol",
      web3_js.getOptionDecoder(web3_js.addDecoderSizePrefix(web3_js.getUtf8Decoder(), web3_js.getU32Decoder()))
    ],
    [
      "uri",
      web3_js.getOptionDecoder(web3_js.addDecoderSizePrefix(web3_js.getUtf8Decoder(), web3_js.getU32Decoder()))
    ],
    ["creators", web3_js.getOptionDecoder(web3_js.getArrayDecoder(getCreatorDecoder()))],
    ["sellerFeeBasisPoints", web3_js.getOptionDecoder(web3_js.getU16Decoder())],
    ["primarySaleHappened", web3_js.getOptionDecoder(web3_js.getBooleanDecoder())],
    ["isMutable", web3_js.getOptionDecoder(web3_js.getBooleanDecoder())]
  ]);
}
function getUpdateMetadataInstructionDataCodec() {
  return web3_js.combineCodec(
    getUpdateMetadataInstructionDataEncoder(),
    getUpdateMetadataInstructionDataDecoder()
  );
}
function getUpdateMetadataInstruction(input) {
  const programAddress = BUBBLEGUM_PROGRAM_ADDRESS;
  const originalAccounts = {
    treeAuthority: { value: input.treeAuthority ?? null, isWritable: false },
    authority: { value: input.authority ?? null, isWritable: false },
    collectionMint: { value: input.collectionMint ?? null, isWritable: false },
    collectionMetadata: {
      value: input.collectionMetadata ?? null,
      isWritable: false
    },
    collectionAuthorityRecordPda: {
      value: input.collectionAuthorityRecordPda ?? null,
      isWritable: false
    },
    leafOwner: { value: input.leafOwner ?? null, isWritable: false },
    leafDelegate: { value: input.leafDelegate ?? null, isWritable: false },
    payer: { value: input.payer ?? null, isWritable: false },
    merkleTree: { value: input.merkleTree ?? null, isWritable: true },
    logWrapper: { value: input.logWrapper ?? null, isWritable: false },
    compressionProgram: {
      value: input.compressionProgram ?? null,
      isWritable: false
    },
    tokenMetadataProgram: {
      value: input.tokenMetadataProgram ?? null,
      isWritable: false
    },
    systemProgram: { value: input.systemProgram ?? null, isWritable: false }
  };
  const accounts = originalAccounts;
  const args = { ...input };
  if (!accounts.tokenMetadataProgram.value) {
    accounts.tokenMetadataProgram.value = "metaqbxxUerdq28cj1RbAWkYQm3ybzjb6a8bt518x1s";
  }
  if (!accounts.systemProgram.value) {
    accounts.systemProgram.value = "11111111111111111111111111111111";
  }
  const getAccountMeta = getAccountMetaFactory2(programAddress);
  const instruction = {
    accounts: [
      getAccountMeta(accounts.treeAuthority),
      getAccountMeta(accounts.authority),
      getAccountMeta(accounts.collectionMint),
      getAccountMeta(accounts.collectionMetadata),
      getAccountMeta(accounts.collectionAuthorityRecordPda),
      getAccountMeta(accounts.leafOwner),
      getAccountMeta(accounts.leafDelegate),
      getAccountMeta(accounts.payer),
      getAccountMeta(accounts.merkleTree),
      getAccountMeta(accounts.logWrapper),
      getAccountMeta(accounts.compressionProgram),
      getAccountMeta(accounts.tokenMetadataProgram),
      getAccountMeta(accounts.systemProgram)
    ],
    programAddress,
    data: getUpdateMetadataInstructionDataEncoder().encode(
      args
    )
  };
  return instruction;
}
function parseUpdateMetadataInstruction(instruction) {
  if (instruction.accounts.length < 13) {
    throw new Error("Not enough accounts");
  }
  let accountIndex = 0;
  const getNextAccount = () => {
    const accountMeta = instruction.accounts[accountIndex];
    accountIndex += 1;
    return accountMeta;
  };
  const getNextOptionalAccount = () => {
    const accountMeta = getNextAccount();
    return accountMeta.address === BUBBLEGUM_PROGRAM_ADDRESS ? void 0 : accountMeta;
  };
  return {
    programAddress: instruction.programAddress,
    accounts: {
      treeAuthority: getNextAccount(),
      authority: getNextAccount(),
      collectionMint: getNextOptionalAccount(),
      collectionMetadata: getNextOptionalAccount(),
      collectionAuthorityRecordPda: getNextOptionalAccount(),
      leafOwner: getNextAccount(),
      leafDelegate: getNextAccount(),
      payer: getNextAccount(),
      merkleTree: getNextAccount(),
      logWrapper: getNextAccount(),
      compressionProgram: getNextAccount(),
      tokenMetadataProgram: getNextAccount(),
      systemProgram: getNextAccount()
    },
    data: getUpdateMetadataInstructionDataDecoder().decode(instruction.data)
  };
}
function getVerifyCollectionInstructionDataEncoder() {
  return web3_js.transformEncoder(
    web3_js.getStructEncoder([
      ["discriminator", web3_js.fixEncoderSize(web3_js.getBytesEncoder(), 8)],
      ["root", web3_js.fixEncoderSize(web3_js.getBytesEncoder(), 32)],
      ["dataHash", web3_js.fixEncoderSize(web3_js.getBytesEncoder(), 32)],
      ["creatorHash", web3_js.fixEncoderSize(web3_js.getBytesEncoder(), 32)],
      ["nonce", web3_js.getU64Encoder()],
      ["index", web3_js.getU32Encoder()],
      ["message", getMetadataArgsEncoder()]
    ]),
    (value) => ({
      ...value,
      discriminator: new Uint8Array([56, 113, 101, 253, 79, 55, 122, 169])
    })
  );
}
function getVerifyCollectionInstructionDataDecoder() {
  return web3_js.getStructDecoder([
    ["discriminator", web3_js.fixDecoderSize(web3_js.getBytesDecoder(), 8)],
    ["root", web3_js.fixDecoderSize(web3_js.getBytesDecoder(), 32)],
    ["dataHash", web3_js.fixDecoderSize(web3_js.getBytesDecoder(), 32)],
    ["creatorHash", web3_js.fixDecoderSize(web3_js.getBytesDecoder(), 32)],
    ["nonce", web3_js.getU64Decoder()],
    ["index", web3_js.getU32Decoder()],
    ["message", getMetadataArgsDecoder()]
  ]);
}
function getVerifyCollectionInstructionDataCodec() {
  return web3_js.combineCodec(
    getVerifyCollectionInstructionDataEncoder(),
    getVerifyCollectionInstructionDataDecoder()
  );
}
function getVerifyCollectionInstruction(input) {
  const programAddress = BUBBLEGUM_PROGRAM_ADDRESS;
  const originalAccounts = {
    treeAuthority: { value: input.treeAuthority ?? null, isWritable: false },
    leafOwner: { value: input.leafOwner ?? null, isWritable: false },
    leafDelegate: { value: input.leafDelegate ?? null, isWritable: false },
    merkleTree: { value: input.merkleTree ?? null, isWritable: true },
    payer: { value: input.payer ?? null, isWritable: false },
    treeDelegate: { value: input.treeDelegate ?? null, isWritable: false },
    collectionAuthority: {
      value: input.collectionAuthority ?? null,
      isWritable: false
    },
    collectionAuthorityRecordPda: {
      value: input.collectionAuthorityRecordPda ?? null,
      isWritable: false
    },
    collectionMint: { value: input.collectionMint ?? null, isWritable: false },
    collectionMetadata: {
      value: input.collectionMetadata ?? null,
      isWritable: true
    },
    editionAccount: { value: input.editionAccount ?? null, isWritable: false },
    bubblegumSigner: {
      value: input.bubblegumSigner ?? null,
      isWritable: false
    },
    logWrapper: { value: input.logWrapper ?? null, isWritable: false },
    compressionProgram: {
      value: input.compressionProgram ?? null,
      isWritable: false
    },
    tokenMetadataProgram: {
      value: input.tokenMetadataProgram ?? null,
      isWritable: false
    },
    systemProgram: { value: input.systemProgram ?? null, isWritable: false }
  };
  const accounts = originalAccounts;
  const args = { ...input };
  if (!accounts.tokenMetadataProgram.value) {
    accounts.tokenMetadataProgram.value = "metaqbxxUerdq28cj1RbAWkYQm3ybzjb6a8bt518x1s";
  }
  if (!accounts.systemProgram.value) {
    accounts.systemProgram.value = "11111111111111111111111111111111";
  }
  const getAccountMeta = getAccountMetaFactory2(programAddress);
  const instruction = {
    accounts: [
      getAccountMeta(accounts.treeAuthority),
      getAccountMeta(accounts.leafOwner),
      getAccountMeta(accounts.leafDelegate),
      getAccountMeta(accounts.merkleTree),
      getAccountMeta(accounts.payer),
      getAccountMeta(accounts.treeDelegate),
      getAccountMeta(accounts.collectionAuthority),
      getAccountMeta(accounts.collectionAuthorityRecordPda),
      getAccountMeta(accounts.collectionMint),
      getAccountMeta(accounts.collectionMetadata),
      getAccountMeta(accounts.editionAccount),
      getAccountMeta(accounts.bubblegumSigner),
      getAccountMeta(accounts.logWrapper),
      getAccountMeta(accounts.compressionProgram),
      getAccountMeta(accounts.tokenMetadataProgram),
      getAccountMeta(accounts.systemProgram)
    ],
    programAddress,
    data: getVerifyCollectionInstructionDataEncoder().encode(
      args
    )
  };
  return instruction;
}
function parseVerifyCollectionInstruction(instruction) {
  if (instruction.accounts.length < 16) {
    throw new Error("Not enough accounts");
  }
  let accountIndex = 0;
  const getNextAccount = () => {
    const accountMeta = instruction.accounts[accountIndex];
    accountIndex += 1;
    return accountMeta;
  };
  return {
    programAddress: instruction.programAddress,
    accounts: {
      treeAuthority: getNextAccount(),
      leafOwner: getNextAccount(),
      leafDelegate: getNextAccount(),
      merkleTree: getNextAccount(),
      payer: getNextAccount(),
      treeDelegate: getNextAccount(),
      collectionAuthority: getNextAccount(),
      collectionAuthorityRecordPda: getNextAccount(),
      collectionMint: getNextAccount(),
      collectionMetadata: getNextAccount(),
      editionAccount: getNextAccount(),
      bubblegumSigner: getNextAccount(),
      logWrapper: getNextAccount(),
      compressionProgram: getNextAccount(),
      tokenMetadataProgram: getNextAccount(),
      systemProgram: getNextAccount()
    },
    data: getVerifyCollectionInstructionDataDecoder().decode(instruction.data)
  };
}
function getVerifyCreatorInstructionDataEncoder() {
  return web3_js.transformEncoder(
    web3_js.getStructEncoder([
      ["discriminator", web3_js.fixEncoderSize(web3_js.getBytesEncoder(), 8)],
      ["root", web3_js.fixEncoderSize(web3_js.getBytesEncoder(), 32)],
      ["dataHash", web3_js.fixEncoderSize(web3_js.getBytesEncoder(), 32)],
      ["creatorHash", web3_js.fixEncoderSize(web3_js.getBytesEncoder(), 32)],
      ["nonce", web3_js.getU64Encoder()],
      ["index", web3_js.getU32Encoder()],
      ["message", getMetadataArgsEncoder()]
    ]),
    (value) => ({
      ...value,
      discriminator: new Uint8Array([52, 17, 96, 132, 71, 4, 85, 194])
    })
  );
}
function getVerifyCreatorInstructionDataDecoder() {
  return web3_js.getStructDecoder([
    ["discriminator", web3_js.fixDecoderSize(web3_js.getBytesDecoder(), 8)],
    ["root", web3_js.fixDecoderSize(web3_js.getBytesDecoder(), 32)],
    ["dataHash", web3_js.fixDecoderSize(web3_js.getBytesDecoder(), 32)],
    ["creatorHash", web3_js.fixDecoderSize(web3_js.getBytesDecoder(), 32)],
    ["nonce", web3_js.getU64Decoder()],
    ["index", web3_js.getU32Decoder()],
    ["message", getMetadataArgsDecoder()]
  ]);
}
function getVerifyCreatorInstructionDataCodec() {
  return web3_js.combineCodec(
    getVerifyCreatorInstructionDataEncoder(),
    getVerifyCreatorInstructionDataDecoder()
  );
}
function getVerifyCreatorInstruction(input) {
  const programAddress = BUBBLEGUM_PROGRAM_ADDRESS;
  const originalAccounts = {
    treeAuthority: { value: input.treeAuthority ?? null, isWritable: false },
    leafOwner: { value: input.leafOwner ?? null, isWritable: false },
    leafDelegate: { value: input.leafDelegate ?? null, isWritable: false },
    merkleTree: { value: input.merkleTree ?? null, isWritable: true },
    payer: { value: input.payer ?? null, isWritable: false },
    creator: { value: input.creator ?? null, isWritable: false },
    logWrapper: { value: input.logWrapper ?? null, isWritable: false },
    compressionProgram: {
      value: input.compressionProgram ?? null,
      isWritable: false
    },
    systemProgram: { value: input.systemProgram ?? null, isWritable: false }
  };
  const accounts = originalAccounts;
  const args = { ...input };
  if (!accounts.systemProgram.value) {
    accounts.systemProgram.value = "11111111111111111111111111111111";
  }
  const getAccountMeta = getAccountMetaFactory2(programAddress);
  const instruction = {
    accounts: [
      getAccountMeta(accounts.treeAuthority),
      getAccountMeta(accounts.leafOwner),
      getAccountMeta(accounts.leafDelegate),
      getAccountMeta(accounts.merkleTree),
      getAccountMeta(accounts.payer),
      getAccountMeta(accounts.creator),
      getAccountMeta(accounts.logWrapper),
      getAccountMeta(accounts.compressionProgram),
      getAccountMeta(accounts.systemProgram)
    ],
    programAddress,
    data: getVerifyCreatorInstructionDataEncoder().encode(
      args
    )
  };
  return instruction;
}
function parseVerifyCreatorInstruction(instruction) {
  if (instruction.accounts.length < 9) {
    throw new Error("Not enough accounts");
  }
  let accountIndex = 0;
  const getNextAccount = () => {
    const accountMeta = instruction.accounts[accountIndex];
    accountIndex += 1;
    return accountMeta;
  };
  return {
    programAddress: instruction.programAddress,
    accounts: {
      treeAuthority: getNextAccount(),
      leafOwner: getNextAccount(),
      leafDelegate: getNextAccount(),
      merkleTree: getNextAccount(),
      payer: getNextAccount(),
      creator: getNextAccount(),
      logWrapper: getNextAccount(),
      compressionProgram: getNextAccount(),
      systemProgram: getNextAccount()
    },
    data: getVerifyCreatorInstructionDataDecoder().decode(instruction.data)
  };
}
var getCanopyDecoderFactory = (canopyDepth) => {
  const canopyBytesSize = Math.max(((1 << canopyDepth + 1) - 2) * 32, 0);
  return web3_js.getStructDecoder(
    [
      ["canopyBytes", web3_js.getArrayDecoder(web3_js.getU8Decoder(), { size: canopyBytesSize })]
    ]
  );
};
function getConcurrentMerkleTreeDecoderFactory(maxDepth, maxBufferSize) {
  return web3_js.getStructDecoder([
    ["sequenceNumber", web3_js.getU64Decoder()],
    ["activeIndex", web3_js.getU64Decoder()],
    ["bufferSize", web3_js.getU64Decoder()],
    [
      "changeLogs",
      web3_js.getArrayDecoder(getChangeLogDecoderFactory(maxDepth), {
        size: maxBufferSize
      })
    ],
    ["rightMostPath", getPathDecoderFactory(maxDepth)]
  ]);
}
function getChangeLogDecoderFactory(maxDepth) {
  return web3_js.getStructDecoder([
    ["root", web3_js.getAddressDecoder()],
    ["pathNodes", web3_js.getArrayDecoder(web3_js.getAddressDecoder(), { size: maxDepth })],
    ["index", web3_js.getU32Decoder()],
    ["_padding", web3_js.getU32Decoder()]
  ]);
}
function getPathDecoderFactory(maxDepth) {
  return web3_js.getStructDecoder([
    ["proof", web3_js.getArrayDecoder(web3_js.getAddressDecoder(), { size: maxDepth })],
    ["leaf", web3_js.getAddressDecoder()],
    ["index", web3_js.getU32Decoder()],
    ["_padding", web3_js.getU32Decoder()]
  ]);
}
function computeCompressedNFTHash(assetId, owner, delegate, treeNonce, metadata) {
  const message = Buffer.concat([
    Buffer.from([1]),
    // All NFTs are version 1 right now
    new Uint8Array(web3_js.getAddressEncoder().encode(assetId)),
    new Uint8Array(web3_js.getAddressEncoder().encode(owner)),
    new Uint8Array(web3_js.getAddressEncoder().encode(delegate)),
    new Uint8Array(web3_js.getU64Encoder().encode(treeNonce)),
    computeDataHash(metadata, new Uint8Array(web3_js.getU16Encoder().encode(metadata.sellerFeeBasisPoints))),
    computeCreatorHash(metadata.creators)
  ]);
  return Buffer.from(jsSha3.keccak_256.digest(message));
}
var computeDataHash = (metadataArgs, sellerFeeBasisPointsBuffer) => Buffer.from(
  jsSha3.keccak_256.digest(
    Buffer.concat([
      computeMetadataArgsHash(metadataArgs),
      sellerFeeBasisPointsBuffer
    ])
  )
);
function computeMetadataArgsHash(metadataArgs) {
  const serializedMetadataArgs = getMetadataArgsEncoder().encode(metadataArgs);
  return Buffer.from(jsSha3.keccak_256.digest(serializedMetadataArgs));
}
function computeCreatorHash(creators) {
  const bufferOfCreatorData = Buffer.concat(
    creators.map((creator) => {
      return Buffer.concat([
        new Uint8Array(web3_js.getAddressEncoder().encode(creator.address)),
        Buffer.from([creator.verified ? 1 : 0]),
        Buffer.from([creator.share])
      ]);
    })
  );
  return Buffer.from(jsSha3.keccak_256.digest(bufferOfCreatorData));
}
function getConcurrentMerkleTreeAccountSize(maxDepth, maxBufferSize, canopyDepth, headerVersion = "V1") {
  if (headerVersion != "V1") {
    throw Error("Unsupported header version");
  }
  return 2 + 54 + // concurrentMerkleTreeHeaderDataV1Beet.byteSize shoule be constant for V1
  getConcurrentMerkleTreeDecoderFactory(maxDepth, maxBufferSize).fixedSize + (canopyDepth ? getCanopyDecoderFactory(canopyDepth).fixedSize : 0);
}
async function findTreeAuthorityPda(seeds, config = {}) {
  const {
    programAddress = "BGUMAp9Gq7iTEuizy4pqaxsTyUCBK68MDfK752saRPUY"
  } = config;
  return await web3_js.getProgramDerivedAddress({
    programAddress,
    seeds: [web3_js.getAddressEncoder().encode(seeds.merkleTree)]
  });
}
var MerkleTree = class {
  leaves;
  root;
  depth;
  constructor(leaves) {
    const [nodes, finalLeaves] = buildLeaves(leaves);
    let seqNum = leaves.length;
    while (nodes.size() > 1) {
      const left = nodes.dequeue();
      const level = left.level;
      let right;
      if (level != nodes.peek().level) {
        right = emptyTreeNode(level, seqNum);
        seqNum++;
      } else {
        right = nodes.dequeue();
      }
      const parent = {
        id: seqNum,
        left,
        level: level + 1,
        node: hash(left.node, right.node),
        parent: void 0,
        right
      };
      left.parent = parent;
      right.parent = parent;
      nodes.enqueue(parent);
      seqNum++;
    }
    this.leaves = finalLeaves;
    this.root = nodes.peek().node;
    this.depth = nodes.peek().level + 1;
  }
  getProof(leafIndex, minimizeProofHeight = false, treeHeight = -1, verbose = false) {
    const proof = [];
    let node = this.leaves[leafIndex];
    let height = 0;
    while (typeof node.parent !== "undefined") {
      if (minimizeProofHeight && height >= treeHeight) {
        break;
      }
      if (verbose) {
        console.log(`${node.level}: ${Uint8Array.from(node.node)}`);
      }
      const parent = node.parent;
      if (parent.left.id === node.id) {
        proof.push(parent.right);
        const hashed = hash(node.node, parent.right.node);
        if (!hashed.equals(parent.node)) {
          console.log(hashed);
          console.log(parent.node);
          throw new Error("Invariant broken when hashing left node");
        }
      } else {
        proof.push(parent.left);
        const hashed = hash(parent.left.node, node.node);
        if (!hashed.equals(parent.node)) {
          console.log(hashed);
          console.log(parent.node);
          throw new Error("Invariant broken when hashing right node");
        }
      }
      node = parent;
      height++;
    }
    return {
      proof: proof.map((treeNode) => treeNode.node)
    };
  }
};
function sparseMerkleTreeFromLeaves(leaves, depth) {
  const _leaves = [];
  for (let i = 0; i < 2 ** depth; i++) {
    if (i < leaves.length) {
      _leaves.push(leaves[i]);
    } else {
      _leaves.push(Buffer.alloc(32));
    }
  }
  return new MerkleTree(_leaves);
}
var CACHE_EMPTY_NODE = /* @__PURE__ */ new Map();
var LEAF_BUFFER_LENGTH = 32;
function emptyNode(level) {
  if (CACHE_EMPTY_NODE.has(level)) {
    return CACHE_EMPTY_NODE.get(level);
  }
  if (level == 0) {
    return Buffer.alloc(32);
  }
  const result = hash(emptyNode(level - 1), emptyNode(level - 1));
  CACHE_EMPTY_NODE.set(level, result);
  return result;
}
function emptyTreeNode(level, id) {
  return {
    id,
    left: void 0,
    level,
    node: emptyNode(level),
    parent: void 0,
    right: void 0
  };
}
function buildLeaves(leaves) {
  const nodes = new Collections__namespace.Queue();
  const finalLeaves = [];
  leaves.forEach((buffer, index) => {
    if (buffer.length != LEAF_BUFFER_LENGTH) {
      throw Error(
        `Provided leaf has length: ${buffer.length}, but we need all leaves to be length ${LEAF_BUFFER_LENGTH}`
      );
    }
    const treeNode = {
      id: index,
      left: void 0,
      level: 0,
      node: buffer,
      parent: void 0,
      right: void 0
    };
    nodes.enqueue(treeNode);
    finalLeaves.push(treeNode);
  });
  return [nodes, finalLeaves];
}
function hash(left, right) {
  return Buffer.from(jsSha3.keccak_256.digest(Buffer.concat([left, right])));
}
var DEFAULT_DEPTH_SIZE = {
  maxDepth: 3,
  maxBufferSize: 8
};
async function setupSingleVerifiedCNFT({
  client,
  cNftOwner,
  payer,
  mintAuthority,
  collectionOwner,
  treeOwner,
  creatorKeypair,
  creator,
  treeDepthSize = DEFAULT_DEPTH_SIZE,
  canopyDepth = 0,
  leafIndex = 0
}) {
  payer = payer ?? creatorKeypair;
  mintAuthority = mintAuthority ?? creatorKeypair;
  collectionOwner = collectionOwner ?? creatorKeypair;
  treeOwner = treeOwner ?? creatorKeypair;
  const index = leafIndex;
  const {
    collectionMint,
    metadata
  } = await mplTokenMetadata.initializeCollection({
    client,
    owner: collectionOwner.address,
    payer,
    mintAuthority,
    creators: [creator]
  });
  const merkleTree = await makeTree({
    client,
    treeOwner,
    canopyDepth,
    depthSizePair: treeDepthSize
  });
  const cnftMetadataArgs = {
    ...metadata,
    collection: web3_js.some({
      key: collectionMint,
      verified: true
    }),
    editionNonce: web3_js.some(0),
    tokenStandard: web3_js.some(0),
    uses: web3_js.none(),
    creators: [creator],
    tokenProgramVersion: 0 /* Original */
  };
  await mintCNft({
    client,
    merkleTree,
    metadata: cnftMetadataArgs,
    treeOwner,
    receiver: cNftOwner,
    unverifiedCollection: false
  });
  const { leaf, assetId } = await makeLeaf({
    index,
    merkleTree,
    metadata: cnftMetadataArgs,
    owner: cNftOwner
  });
  await verifyCNftCreator({
    client,
    index,
    merkleTree,
    metadata: cnftMetadataArgs,
    owner: cNftOwner,
    payer,
    verifiedCreator: creatorKeypair
  });
  await verifyCNft({
    client,
    index,
    merkleTree,
    metadata: cnftMetadataArgs,
    owner: cNftOwner,
    payer
  });
  const memTree = sparseMerkleTreeFromLeaves(
    [leaf],
    treeDepthSize.maxDepth
  );
  const proof = memTree.getProof(index, false, treeDepthSize.maxDepth, false);
  return {
    collectionMint,
    merkleTree,
    memTree,
    root: memTree.root,
    meta: cnftMetadataArgs,
    leaf,
    assetId,
    proof: proof.proof.map((proof2) => {
      return web3_js.getAddressDecoder().decode(new Uint8Array(proof2));
    })
  };
}
var makeLeaf = async ({
  index,
  owner,
  delegate,
  merkleTree,
  metadata
}) => {
  const nonce = BigInt(index);
  const [assetId] = await findAssetIdPda({ merkleTree, leafIndex: nonce });
  const leaf = computeCompressedNFTHash(
    assetId,
    owner,
    delegate ?? owner,
    nonce,
    metadata
  );
  return {
    leaf,
    assetId
  };
};
async function findAssetIdPda(seeds, config = {}) {
  const {
    programAddress = "BGUMAp9Gq7iTEuizy4pqaxsTyUCBK68MDfK752saRPUY"
  } = config;
  return await web3_js.getProgramDerivedAddress({
    programAddress,
    seeds: [
      web3_js.getUtf8Encoder().encode("asset"),
      web3_js.getAddressEncoder().encode(seeds.merkleTree),
      web3_js.getU64Encoder().encode(seeds.leafIndex)
    ]
  });
}
var mintCNft = async ({
  client,
  treeOwner,
  receiver,
  metadata,
  merkleTree,
  unverifiedCollection = false
}) => {
  const [bgumSigner, __] = await web3_js.getProgramDerivedAddress({
    programAddress: testHelpers.MPL_BUBBLEGUM_PROGRAM_ID,
    seeds: [Buffer.from("collection_cpi")]
  });
  const [treeAuthority] = await findTreeAuthorityPda({ merkleTree });
  const collectionKey = web3_js.unwrapOption(metadata.collection).key;
  const mintIx = !!metadata.collection && !unverifiedCollection ? getMintToCollectionV1Instruction(
    {
      merkleTree,
      treeAuthority,
      treeDelegate: treeOwner,
      payer: treeOwner,
      leafDelegate: receiver,
      leafOwner: receiver,
      compressionProgram: testHelpers.ACCOUNT_COMPRESSION_PROGRAM_ID,
      logWrapper: testHelpers.NOOP_PROGRAM_ID,
      bubblegumSigner: bgumSigner,
      collectionAuthority: treeOwner,
      collectionAuthorityRecordPda: testHelpers.MPL_BUBBLEGUM_PROGRAM_ID,
      collectionMetadata: await getMetadata(collectionKey).then((resp) => resp[0]),
      collectionMint: collectionKey,
      editionAccount: await getMasterEdition(collectionKey).then((resp) => resp[0]),
      tokenMetadataProgram: testHelpers.MPL_TOKEN_METADATA_PROGRAM_ID,
      metadataArgs: {
        ...metadata
      }
    }
  ) : getMintV1Instruction(
    {
      merkleTree,
      treeAuthority,
      treeDelegate: treeOwner,
      payer: treeOwner,
      leafDelegate: receiver,
      leafOwner: receiver,
      compressionProgram: testHelpers.ACCOUNT_COMPRESSION_PROGRAM_ID,
      logWrapper: testHelpers.NOOP_PROGRAM_ID,
      message: metadata
    }
  );
  const { value: latestBlockhash } = await client.rpc.getLatestBlockhash().send();
  await web3_js.pipe(
    web3_js.createTransactionMessage({ version: 0 }),
    (tx) => web3_js.setTransactionMessageFeePayerSigner(treeOwner, tx),
    (tx) => web3_js.setTransactionMessageLifetimeUsingBlockhash(latestBlockhash, tx),
    (tx) => web3_js.appendTransactionMessageInstruction(mintIx, tx),
    (tx) => testHelpers.signAndSendTransaction(client, tx)
  );
};
async function getMetadata(mint) {
  return await web3_js.getProgramDerivedAddress({
    seeds: [Buffer.from("metadata"), web3_js.getAddressEncoder().encode(testHelpers.MPL_TOKEN_METADATA_PROGRAM_ID), web3_js.getAddressEncoder().encode(mint)],
    programAddress: testHelpers.MPL_TOKEN_METADATA_PROGRAM_ID
  });
}
async function getMasterEdition(mint) {
  return await web3_js.getProgramDerivedAddress({
    seeds: [
      Buffer.from("metadata"),
      web3_js.getAddressEncoder().encode(testHelpers.MPL_TOKEN_METADATA_PROGRAM_ID),
      web3_js.getAddressEncoder().encode(mint),
      Buffer.from("edition")
    ],
    programAddress: testHelpers.MPL_TOKEN_METADATA_PROGRAM_ID
  });
}
var DEFAULT_DEPTH_SIZE2 = {
  maxDepth: 14,
  maxBufferSize: 64
};
var makeTree = async ({
  client,
  treeOwner,
  depthSizePair = DEFAULT_DEPTH_SIZE2,
  canopyDepth = 0
}) => {
  const merkleTreeKeypair = await web3_js.generateKeyPairSigner();
  const merkleTree = merkleTreeKeypair.address;
  const space = getConcurrentMerkleTreeAccountSize(
    depthSizePair.maxDepth,
    depthSizePair.maxBufferSize,
    canopyDepth
  );
  const allocTreeIx = system.getCreateAccountInstruction({
    payer: treeOwner,
    newAccount: merkleTreeKeypair,
    lamports: await client.rpc.getMinimumBalanceForRentExemption(BigInt(space)).send(),
    space,
    programAddress: testHelpers.ACCOUNT_COMPRESSION_PROGRAM_ID
  });
  const [treeAuthority, _bump] = await findTreeAuthorityPda({ merkleTree });
  const createTreeIx = getCreateTreeInstruction(
    {
      merkleTree,
      treeAuthority,
      treeCreator: treeOwner,
      payer: treeOwner,
      logWrapper: testHelpers.NOOP_PROGRAM_ID,
      compressionProgram: testHelpers.ACCOUNT_COMPRESSION_PROGRAM_ID,
      maxBufferSize: depthSizePair.maxBufferSize,
      maxDepth: depthSizePair.maxDepth,
      public: false
    }
  );
  const { value: latestBlockhash } = await client.rpc.getLatestBlockhash().send();
  await web3_js.pipe(
    web3_js.createTransactionMessage({ version: 0 }),
    (tx) => web3_js.setTransactionMessageFeePayerSigner(merkleTreeKeypair, tx),
    (tx) => web3_js.setTransactionMessageFeePayerSigner(treeOwner, tx),
    (tx) => web3_js.setTransactionMessageLifetimeUsingBlockhash(latestBlockhash, tx),
    (tx) => web3_js.appendTransactionMessageInstruction(allocTreeIx, tx),
    (tx) => web3_js.appendTransactionMessageInstruction(createTreeIx, tx),
    (tx) => testHelpers.signAndSendTransaction(client, tx)
  );
  return merkleTree;
};
async function getRootFromMerkleTreeAddress(client, merkleTree) {
  const merkleTreeData = await client.rpc.getAccountInfo(merkleTree, { encoding: "base64" }).send().then((result) => result.value.data[0]);
  const merkleTreeDataBytes = Buffer.from(merkleTreeData, "base64");
  var currOffset = 0;
  const [merkleTreeHeader, headerOffset] = getConcurrentMerkleTreeHeaderDecoder().read(
    merkleTreeDataBytes,
    currOffset
  );
  currOffset = headerOffset;
  const { fields } = merkleTreeHeader.header;
  const { maxDepth, maxBufferSize } = fields[0];
  const [treeBody] = getConcurrentMerkleTreeDecoderFactory(
    maxDepth,
    maxBufferSize
  ).read(merkleTreeDataBytes, currOffset);
  const activeIndex = Number(treeBody.activeIndex);
  return treeBody.changeLogs[activeIndex].root;
}
var verifyCNftCreator = async ({
  client,
  index,
  owner,
  payer,
  merkleTree,
  metadata,
  verifiedCreator
}) => {
  const root = new Uint8Array(
    web3_js.getAddressEncoder().encode(
      await getRootFromMerkleTreeAddress(client, merkleTree)
    )
  );
  const [treeAuthority] = await findTreeAuthorityPda({ merkleTree });
  const verifyCreatorIx = getVerifyCreatorInstruction(
    {
      merkleTree,
      treeAuthority,
      leafOwner: owner,
      leafDelegate: owner,
      payer,
      creator: verifiedCreator,
      logWrapper: testHelpers.NOOP_PROGRAM_ID,
      compressionProgram: testHelpers.ACCOUNT_COMPRESSION_PROGRAM_ID,
      systemProgram: system.SYSTEM_PROGRAM_ADDRESS,
      root,
      creatorHash: computeCreatorHash(metadata.creators),
      dataHash: computeDataHash(metadata, new Uint8Array(web3_js.getU16Encoder().encode(metadata.sellerFeeBasisPoints))),
      index,
      message: metadata,
      nonce: index
    }
  );
  const { value: latestBlockhash } = await client.rpc.getLatestBlockhash().send();
  await web3_js.pipe(
    web3_js.createTransactionMessage({ version: 0 }),
    (tx) => web3_js.setTransactionMessageFeePayerSigner(verifiedCreator, tx),
    (tx) => web3_js.setTransactionMessageFeePayerSigner(payer, tx),
    (tx) => web3_js.setTransactionMessageLifetimeUsingBlockhash(latestBlockhash, tx),
    (tx) => web3_js.appendTransactionMessageInstruction(verifyCreatorIx, tx),
    (tx) => testHelpers.signAndSendTransaction(client, tx)
  );
  return;
};
var verifyCNft = async ({
  client,
  index,
  owner,
  payer,
  delegate,
  merkleTree,
  metadata
}) => {
  const root = new Uint8Array(
    web3_js.getAddressEncoder().encode(
      await getRootFromMerkleTreeAddress(client, merkleTree)
    )
  );
  const { leaf, assetId } = await makeLeaf({
    index,
    owner,
    delegate,
    merkleTree,
    metadata
  });
  const verifyLeafIx = getVerifyLeafInstruction({
    merkleTree,
    root,
    leaf: new Uint8Array(leaf),
    index
  });
  const cuLimitIx = computeBudget.getSetComputeUnitLimitInstruction({
    units: 4e5
  });
  const { value: latestBlockhash } = await client.rpc.getLatestBlockhash().send();
  await web3_js.pipe(
    web3_js.createTransactionMessage({ version: 0 }),
    (tx) => web3_js.setTransactionMessageFeePayerSigner(payer, tx),
    (tx) => web3_js.setTransactionMessageLifetimeUsingBlockhash(latestBlockhash, tx),
    (tx) => web3_js.appendTransactionMessageInstruction(cuLimitIx, tx),
    (tx) => web3_js.appendTransactionMessageInstruction(verifyLeafIx, tx),
    (tx) => testHelpers.signAndSendTransaction(client, tx)
  );
  return { leaf, assetId };
};

exports.BUBBLEGUM_ERROR__ALREADY_UNVERIFIED = BUBBLEGUM_ERROR__ALREADY_UNVERIFIED;
exports.BUBBLEGUM_ERROR__ALREADY_VERIFIED = BUBBLEGUM_ERROR__ALREADY_VERIFIED;
exports.BUBBLEGUM_ERROR__ASSET_OWNER_MISMATCH = BUBBLEGUM_ERROR__ASSET_OWNER_MISMATCH;
exports.BUBBLEGUM_ERROR__COLLECTION_CANNOT_BE_VERIFIED_IN_THIS_INSTRUCTION = BUBBLEGUM_ERROR__COLLECTION_CANNOT_BE_VERIFIED_IN_THIS_INSTRUCTION;
exports.BUBBLEGUM_ERROR__COLLECTION_MASTER_EDITION_ACCOUNT_INVALID = BUBBLEGUM_ERROR__COLLECTION_MASTER_EDITION_ACCOUNT_INVALID;
exports.BUBBLEGUM_ERROR__COLLECTION_MISMATCH = BUBBLEGUM_ERROR__COLLECTION_MISMATCH;
exports.BUBBLEGUM_ERROR__COLLECTION_MUST_BE_A_UNIQUE_MASTER_EDITION = BUBBLEGUM_ERROR__COLLECTION_MUST_BE_A_UNIQUE_MASTER_EDITION;
exports.BUBBLEGUM_ERROR__COLLECTION_MUST_BE_SIZED = BUBBLEGUM_ERROR__COLLECTION_MUST_BE_SIZED;
exports.BUBBLEGUM_ERROR__COLLECTION_NOT_FOUND = BUBBLEGUM_ERROR__COLLECTION_NOT_FOUND;
exports.BUBBLEGUM_ERROR__CREATORS_TOO_LONG = BUBBLEGUM_ERROR__CREATORS_TOO_LONG;
exports.BUBBLEGUM_ERROR__CREATOR_DID_NOT_UNVERIFY = BUBBLEGUM_ERROR__CREATOR_DID_NOT_UNVERIFY;
exports.BUBBLEGUM_ERROR__CREATOR_DID_NOT_VERIFY = BUBBLEGUM_ERROR__CREATOR_DID_NOT_VERIFY;
exports.BUBBLEGUM_ERROR__CREATOR_HASH_MISMATCH = BUBBLEGUM_ERROR__CREATOR_HASH_MISMATCH;
exports.BUBBLEGUM_ERROR__CREATOR_NOT_FOUND = BUBBLEGUM_ERROR__CREATOR_NOT_FOUND;
exports.BUBBLEGUM_ERROR__CREATOR_SHARE_TOTAL_MUST_BE100 = BUBBLEGUM_ERROR__CREATOR_SHARE_TOTAL_MUST_BE100;
exports.BUBBLEGUM_ERROR__DATA_HASH_MISMATCH = BUBBLEGUM_ERROR__DATA_HASH_MISMATCH;
exports.BUBBLEGUM_ERROR__DECOMPRESSION_DISABLED = BUBBLEGUM_ERROR__DECOMPRESSION_DISABLED;
exports.BUBBLEGUM_ERROR__DUPLICATE_CREATOR_ADDRESS = BUBBLEGUM_ERROR__DUPLICATE_CREATOR_ADDRESS;
exports.BUBBLEGUM_ERROR__HASHING_MISMATCH = BUBBLEGUM_ERROR__HASHING_MISMATCH;
exports.BUBBLEGUM_ERROR__INCORRECT_OWNER = BUBBLEGUM_ERROR__INCORRECT_OWNER;
exports.BUBBLEGUM_ERROR__INSUFFICIENT_MINT_CAPACITY = BUBBLEGUM_ERROR__INSUFFICIENT_MINT_CAPACITY;
exports.BUBBLEGUM_ERROR__INVALID_CANOPY_SIZE = BUBBLEGUM_ERROR__INVALID_CANOPY_SIZE;
exports.BUBBLEGUM_ERROR__INVALID_COLLECTION_AUTHORITY = BUBBLEGUM_ERROR__INVALID_COLLECTION_AUTHORITY;
exports.BUBBLEGUM_ERROR__INVALID_DELEGATE_RECORD = BUBBLEGUM_ERROR__INVALID_DELEGATE_RECORD;
exports.BUBBLEGUM_ERROR__INVALID_TOKEN_STANDARD = BUBBLEGUM_ERROR__INVALID_TOKEN_STANDARD;
exports.BUBBLEGUM_ERROR__LEAF_AUTHORITY_MUST_SIGN = BUBBLEGUM_ERROR__LEAF_AUTHORITY_MUST_SIGN;
exports.BUBBLEGUM_ERROR__METADATA_BASIS_POINTS_TOO_HIGH = BUBBLEGUM_ERROR__METADATA_BASIS_POINTS_TOO_HIGH;
exports.BUBBLEGUM_ERROR__METADATA_IMMUTABLE = BUBBLEGUM_ERROR__METADATA_IMMUTABLE;
exports.BUBBLEGUM_ERROR__METADATA_MINT_MISMATCH = BUBBLEGUM_ERROR__METADATA_MINT_MISMATCH;
exports.BUBBLEGUM_ERROR__METADATA_NAME_TOO_LONG = BUBBLEGUM_ERROR__METADATA_NAME_TOO_LONG;
exports.BUBBLEGUM_ERROR__METADATA_SYMBOL_TOO_LONG = BUBBLEGUM_ERROR__METADATA_SYMBOL_TOO_LONG;
exports.BUBBLEGUM_ERROR__METADATA_URI_TOO_LONG = BUBBLEGUM_ERROR__METADATA_URI_TOO_LONG;
exports.BUBBLEGUM_ERROR__MISSING_COLLECTION_METADATA_ACCOUNT = BUBBLEGUM_ERROR__MISSING_COLLECTION_METADATA_ACCOUNT;
exports.BUBBLEGUM_ERROR__MISSING_COLLECTION_MINT_ACCOUNT = BUBBLEGUM_ERROR__MISSING_COLLECTION_MINT_ACCOUNT;
exports.BUBBLEGUM_ERROR__NO_CREATORS_PRESENT = BUBBLEGUM_ERROR__NO_CREATORS_PRESENT;
exports.BUBBLEGUM_ERROR__NUMERICAL_OVERFLOW_ERROR = BUBBLEGUM_ERROR__NUMERICAL_OVERFLOW_ERROR;
exports.BUBBLEGUM_ERROR__PRIMARY_SALE_CAN_ONLY_BE_FLIPPED_TO_TRUE = BUBBLEGUM_ERROR__PRIMARY_SALE_CAN_ONLY_BE_FLIPPED_TO_TRUE;
exports.BUBBLEGUM_ERROR__PUBLIC_KEY_MISMATCH = BUBBLEGUM_ERROR__PUBLIC_KEY_MISMATCH;
exports.BUBBLEGUM_ERROR__TREE_AUTHORITY_INCORRECT = BUBBLEGUM_ERROR__TREE_AUTHORITY_INCORRECT;
exports.BUBBLEGUM_ERROR__UNKNOWN_EXTERNAL_ERROR = BUBBLEGUM_ERROR__UNKNOWN_EXTERNAL_ERROR;
exports.BUBBLEGUM_ERROR__UNSUPPORTED_SCHEMA_VERSION = BUBBLEGUM_ERROR__UNSUPPORTED_SCHEMA_VERSION;
exports.BUBBLEGUM_ERROR__UPDATE_AUTHORITY_INCORRECT = BUBBLEGUM_ERROR__UPDATE_AUTHORITY_INCORRECT;
exports.BUBBLEGUM_PROGRAM_ADDRESS = BUBBLEGUM_PROGRAM_ADDRESS;
exports.BubblegumAccount = BubblegumAccount;
exports.BubblegumEventType = BubblegumEventType;
exports.BubblegumInstruction = BubblegumInstruction;
exports.CompressionAccountType = CompressionAccountType;
exports.DecompressibleState = DecompressibleState;
exports.InstructionName = InstructionName;
exports.MerkleTree = MerkleTree;
exports.SPL_ACCOUNT_COMPRESSION_ERROR__CANOPY_LENGTH_MISMATCH = SPL_ACCOUNT_COMPRESSION_ERROR__CANOPY_LENGTH_MISMATCH;
exports.SPL_ACCOUNT_COMPRESSION_ERROR__CONCURRENT_MERKLE_TREE_CONSTANTS_ERROR = SPL_ACCOUNT_COMPRESSION_ERROR__CONCURRENT_MERKLE_TREE_CONSTANTS_ERROR;
exports.SPL_ACCOUNT_COMPRESSION_ERROR__CONCURRENT_MERKLE_TREE_ERROR = SPL_ACCOUNT_COMPRESSION_ERROR__CONCURRENT_MERKLE_TREE_ERROR;
exports.SPL_ACCOUNT_COMPRESSION_ERROR__INCORRECT_ACCOUNT_OWNER = SPL_ACCOUNT_COMPRESSION_ERROR__INCORRECT_ACCOUNT_OWNER;
exports.SPL_ACCOUNT_COMPRESSION_ERROR__INCORRECT_ACCOUNT_TYPE = SPL_ACCOUNT_COMPRESSION_ERROR__INCORRECT_ACCOUNT_TYPE;
exports.SPL_ACCOUNT_COMPRESSION_ERROR__INCORRECT_AUTHORITY = SPL_ACCOUNT_COMPRESSION_ERROR__INCORRECT_AUTHORITY;
exports.SPL_ACCOUNT_COMPRESSION_ERROR__INCORRECT_LEAF_LENGTH = SPL_ACCOUNT_COMPRESSION_ERROR__INCORRECT_LEAF_LENGTH;
exports.SPL_ACCOUNT_COMPRESSION_ERROR__LEAF_INDEX_OUT_OF_BOUNDS = SPL_ACCOUNT_COMPRESSION_ERROR__LEAF_INDEX_OUT_OF_BOUNDS;
exports.SPL_ACCOUNT_COMPRESSION_ERROR__ZERO_COPY_ERROR = SPL_ACCOUNT_COMPRESSION_ERROR__ZERO_COPY_ERROR;
exports.SPL_ACCOUNT_COMPRESSION_PROGRAM_ADDRESS = SPL_ACCOUNT_COMPRESSION_PROGRAM_ADDRESS;
exports.SplAccountCompressionInstruction = SplAccountCompressionInstruction;
exports.TokenProgramVersion = TokenProgramVersion2;
exports.TokenStandard = TokenStandard2;
exports.UseMethod = UseMethod;
exports.Version = Version;
exports.accountCompressionEvent = accountCompressionEvent;
exports.applicationDataEvent = applicationDataEvent;
exports.changeLogEvent = changeLogEvent;
exports.computeCompressedNFTHash = computeCompressedNFTHash;
exports.computeCreatorHash = computeCreatorHash;
exports.computeDataHash = computeDataHash;
exports.computeMetadataArgsHash = computeMetadataArgsHash;
exports.concurrentMerkleTreeHeaderData = concurrentMerkleTreeHeaderData;
exports.decodeTreeConfig = decodeTreeConfig;
exports.decodeVoucher = decodeVoucher;
exports.fetchAllMaybeTreeConfig = fetchAllMaybeTreeConfig;
exports.fetchAllMaybeVoucher = fetchAllMaybeVoucher;
exports.fetchAllTreeConfig = fetchAllTreeConfig;
exports.fetchAllVoucher = fetchAllVoucher;
exports.fetchMaybeTreeConfig = fetchMaybeTreeConfig;
exports.fetchMaybeVoucher = fetchMaybeVoucher;
exports.fetchTreeConfig = fetchTreeConfig;
exports.fetchVoucher = fetchVoucher;
exports.findAssetIdPda = findAssetIdPda;
exports.findTreeAuthorityPda = findTreeAuthorityPda;
exports.getAccountCompressionEventCodec = getAccountCompressionEventCodec;
exports.getAccountCompressionEventDecoder = getAccountCompressionEventDecoder;
exports.getAccountCompressionEventEncoder = getAccountCompressionEventEncoder;
exports.getAppendInstruction = getAppendInstruction;
exports.getAppendInstructionDataCodec = getAppendInstructionDataCodec;
exports.getAppendInstructionDataDecoder = getAppendInstructionDataDecoder;
exports.getAppendInstructionDataEncoder = getAppendInstructionDataEncoder;
exports.getApplicationDataEventCodec = getApplicationDataEventCodec;
exports.getApplicationDataEventDecoder = getApplicationDataEventDecoder;
exports.getApplicationDataEventEncoder = getApplicationDataEventEncoder;
exports.getApplicationDataEventV1Codec = getApplicationDataEventV1Codec;
exports.getApplicationDataEventV1Decoder = getApplicationDataEventV1Decoder;
exports.getApplicationDataEventV1Encoder = getApplicationDataEventV1Encoder;
exports.getBubblegumErrorMessage = getBubblegumErrorMessage;
exports.getBubblegumEventTypeCodec = getBubblegumEventTypeCodec;
exports.getBubblegumEventTypeDecoder = getBubblegumEventTypeDecoder;
exports.getBubblegumEventTypeEncoder = getBubblegumEventTypeEncoder;
exports.getBurnInstruction = getBurnInstruction;
exports.getBurnInstructionDataCodec = getBurnInstructionDataCodec;
exports.getBurnInstructionDataDecoder = getBurnInstructionDataDecoder;
exports.getBurnInstructionDataEncoder = getBurnInstructionDataEncoder;
exports.getCancelRedeemInstruction = getCancelRedeemInstruction;
exports.getCancelRedeemInstructionDataCodec = getCancelRedeemInstructionDataCodec;
exports.getCancelRedeemInstructionDataDecoder = getCancelRedeemInstructionDataDecoder;
exports.getCancelRedeemInstructionDataEncoder = getCancelRedeemInstructionDataEncoder;
exports.getCanopyDecoderFactory = getCanopyDecoderFactory;
exports.getChangeLogEventCodec = getChangeLogEventCodec;
exports.getChangeLogEventDecoder = getChangeLogEventDecoder;
exports.getChangeLogEventEncoder = getChangeLogEventEncoder;
exports.getChangeLogEventV1Codec = getChangeLogEventV1Codec;
exports.getChangeLogEventV1Decoder = getChangeLogEventV1Decoder;
exports.getChangeLogEventV1Encoder = getChangeLogEventV1Encoder;
exports.getCloseEmptyTreeInstruction = getCloseEmptyTreeInstruction;
exports.getCloseEmptyTreeInstructionDataCodec = getCloseEmptyTreeInstructionDataCodec;
exports.getCloseEmptyTreeInstructionDataDecoder = getCloseEmptyTreeInstructionDataDecoder;
exports.getCloseEmptyTreeInstructionDataEncoder = getCloseEmptyTreeInstructionDataEncoder;
exports.getCollectionCodec = getCollectionCodec;
exports.getCollectionDecoder = getCollectionDecoder;
exports.getCollectionEncoder = getCollectionEncoder;
exports.getCompressInstruction = getCompressInstruction;
exports.getCompressInstructionDataCodec = getCompressInstructionDataCodec;
exports.getCompressInstructionDataDecoder = getCompressInstructionDataDecoder;
exports.getCompressInstructionDataEncoder = getCompressInstructionDataEncoder;
exports.getCompressionAccountTypeCodec = getCompressionAccountTypeCodec;
exports.getCompressionAccountTypeDecoder = getCompressionAccountTypeDecoder;
exports.getCompressionAccountTypeEncoder = getCompressionAccountTypeEncoder;
exports.getConcurrentMerkleTreeAccountSize = getConcurrentMerkleTreeAccountSize;
exports.getConcurrentMerkleTreeDecoderFactory = getConcurrentMerkleTreeDecoderFactory;
exports.getConcurrentMerkleTreeHeaderCodec = getConcurrentMerkleTreeHeaderCodec;
exports.getConcurrentMerkleTreeHeaderDataCodec = getConcurrentMerkleTreeHeaderDataCodec;
exports.getConcurrentMerkleTreeHeaderDataDecoder = getConcurrentMerkleTreeHeaderDataDecoder;
exports.getConcurrentMerkleTreeHeaderDataEncoder = getConcurrentMerkleTreeHeaderDataEncoder;
exports.getConcurrentMerkleTreeHeaderDataV1Codec = getConcurrentMerkleTreeHeaderDataV1Codec;
exports.getConcurrentMerkleTreeHeaderDataV1Decoder = getConcurrentMerkleTreeHeaderDataV1Decoder;
exports.getConcurrentMerkleTreeHeaderDataV1Encoder = getConcurrentMerkleTreeHeaderDataV1Encoder;
exports.getConcurrentMerkleTreeHeaderDecoder = getConcurrentMerkleTreeHeaderDecoder;
exports.getConcurrentMerkleTreeHeaderEncoder = getConcurrentMerkleTreeHeaderEncoder;
exports.getCreateTreeInstruction = getCreateTreeInstruction;
exports.getCreateTreeInstructionDataCodec = getCreateTreeInstructionDataCodec;
exports.getCreateTreeInstructionDataDecoder = getCreateTreeInstructionDataDecoder;
exports.getCreateTreeInstructionDataEncoder = getCreateTreeInstructionDataEncoder;
exports.getCreatorCodec = getCreatorCodec;
exports.getCreatorDecoder = getCreatorDecoder;
exports.getCreatorEncoder = getCreatorEncoder;
exports.getDecompressV1Instruction = getDecompressV1Instruction;
exports.getDecompressV1InstructionDataCodec = getDecompressV1InstructionDataCodec;
exports.getDecompressV1InstructionDataDecoder = getDecompressV1InstructionDataDecoder;
exports.getDecompressV1InstructionDataEncoder = getDecompressV1InstructionDataEncoder;
exports.getDecompressibleStateCodec = getDecompressibleStateCodec;
exports.getDecompressibleStateDecoder = getDecompressibleStateDecoder;
exports.getDecompressibleStateEncoder = getDecompressibleStateEncoder;
exports.getDelegateInstruction = getDelegateInstruction;
exports.getDelegateInstructionDataCodec = getDelegateInstructionDataCodec;
exports.getDelegateInstructionDataDecoder = getDelegateInstructionDataDecoder;
exports.getDelegateInstructionDataEncoder = getDelegateInstructionDataEncoder;
exports.getInitEmptyMerkleTreeInstruction = getInitEmptyMerkleTreeInstruction;
exports.getInitEmptyMerkleTreeInstructionDataCodec = getInitEmptyMerkleTreeInstructionDataCodec;
exports.getInitEmptyMerkleTreeInstructionDataDecoder = getInitEmptyMerkleTreeInstructionDataDecoder;
exports.getInitEmptyMerkleTreeInstructionDataEncoder = getInitEmptyMerkleTreeInstructionDataEncoder;
exports.getInsertOrAppendInstruction = getInsertOrAppendInstruction;
exports.getInsertOrAppendInstructionDataCodec = getInsertOrAppendInstructionDataCodec;
exports.getInsertOrAppendInstructionDataDecoder = getInsertOrAppendInstructionDataDecoder;
exports.getInsertOrAppendInstructionDataEncoder = getInsertOrAppendInstructionDataEncoder;
exports.getInstructionNameCodec = getInstructionNameCodec;
exports.getInstructionNameDecoder = getInstructionNameDecoder;
exports.getInstructionNameEncoder = getInstructionNameEncoder;
exports.getLeafSchemaCodec = getLeafSchemaCodec;
exports.getLeafSchemaDecoder = getLeafSchemaDecoder;
exports.getLeafSchemaEncoder = getLeafSchemaEncoder;
exports.getMetadataArgsCodec = getMetadataArgsCodec;
exports.getMetadataArgsDecoder = getMetadataArgsDecoder;
exports.getMetadataArgsEncoder = getMetadataArgsEncoder;
exports.getMintToCollectionV1Instruction = getMintToCollectionV1Instruction;
exports.getMintToCollectionV1InstructionDataCodec = getMintToCollectionV1InstructionDataCodec;
exports.getMintToCollectionV1InstructionDataDecoder = getMintToCollectionV1InstructionDataDecoder;
exports.getMintToCollectionV1InstructionDataEncoder = getMintToCollectionV1InstructionDataEncoder;
exports.getMintV1Instruction = getMintV1Instruction;
exports.getMintV1InstructionDataCodec = getMintV1InstructionDataCodec;
exports.getMintV1InstructionDataDecoder = getMintV1InstructionDataDecoder;
exports.getMintV1InstructionDataEncoder = getMintV1InstructionDataEncoder;
exports.getPathNodeCodec = getPathNodeCodec;
exports.getPathNodeDecoder = getPathNodeDecoder;
exports.getPathNodeEncoder = getPathNodeEncoder;
exports.getRedeemInstruction = getRedeemInstruction;
exports.getRedeemInstructionDataCodec = getRedeemInstructionDataCodec;
exports.getRedeemInstructionDataDecoder = getRedeemInstructionDataDecoder;
exports.getRedeemInstructionDataEncoder = getRedeemInstructionDataEncoder;
exports.getReplaceLeafInstruction = getReplaceLeafInstruction;
exports.getReplaceLeafInstructionDataCodec = getReplaceLeafInstructionDataCodec;
exports.getReplaceLeafInstructionDataDecoder = getReplaceLeafInstructionDataDecoder;
exports.getReplaceLeafInstructionDataEncoder = getReplaceLeafInstructionDataEncoder;
exports.getRootFromMerkleTreeAddress = getRootFromMerkleTreeAddress;
exports.getSetAndVerifyCollectionInstruction = getSetAndVerifyCollectionInstruction;
exports.getSetAndVerifyCollectionInstructionDataCodec = getSetAndVerifyCollectionInstructionDataCodec;
exports.getSetAndVerifyCollectionInstructionDataDecoder = getSetAndVerifyCollectionInstructionDataDecoder;
exports.getSetAndVerifyCollectionInstructionDataEncoder = getSetAndVerifyCollectionInstructionDataEncoder;
exports.getSetDecompressableStateInstruction = getSetDecompressableStateInstruction;
exports.getSetDecompressableStateInstructionDataCodec = getSetDecompressableStateInstructionDataCodec;
exports.getSetDecompressableStateInstructionDataDecoder = getSetDecompressableStateInstructionDataDecoder;
exports.getSetDecompressableStateInstructionDataEncoder = getSetDecompressableStateInstructionDataEncoder;
exports.getSetDecompressibleStateInstruction = getSetDecompressibleStateInstruction;
exports.getSetDecompressibleStateInstructionDataCodec = getSetDecompressibleStateInstructionDataCodec;
exports.getSetDecompressibleStateInstructionDataDecoder = getSetDecompressibleStateInstructionDataDecoder;
exports.getSetDecompressibleStateInstructionDataEncoder = getSetDecompressibleStateInstructionDataEncoder;
exports.getSetTreeDelegateInstruction = getSetTreeDelegateInstruction;
exports.getSetTreeDelegateInstructionDataCodec = getSetTreeDelegateInstructionDataCodec;
exports.getSetTreeDelegateInstructionDataDecoder = getSetTreeDelegateInstructionDataDecoder;
exports.getSetTreeDelegateInstructionDataEncoder = getSetTreeDelegateInstructionDataEncoder;
exports.getSplAccountCompressionErrorMessage = getSplAccountCompressionErrorMessage;
exports.getTokenProgramVersionCodec = getTokenProgramVersionCodec;
exports.getTokenProgramVersionDecoder = getTokenProgramVersionDecoder;
exports.getTokenProgramVersionEncoder = getTokenProgramVersionEncoder;
exports.getTokenStandardCodec = getTokenStandardCodec;
exports.getTokenStandardDecoder = getTokenStandardDecoder;
exports.getTokenStandardEncoder = getTokenStandardEncoder;
exports.getTransferAuthorityInstruction = getTransferAuthorityInstruction;
exports.getTransferAuthorityInstructionDataCodec = getTransferAuthorityInstructionDataCodec;
exports.getTransferAuthorityInstructionDataDecoder = getTransferAuthorityInstructionDataDecoder;
exports.getTransferAuthorityInstructionDataEncoder = getTransferAuthorityInstructionDataEncoder;
exports.getTransferInstruction = getTransferInstruction;
exports.getTransferInstructionDataCodec = getTransferInstructionDataCodec;
exports.getTransferInstructionDataDecoder = getTransferInstructionDataDecoder;
exports.getTransferInstructionDataEncoder = getTransferInstructionDataEncoder;
exports.getTreeConfigCodec = getTreeConfigCodec;
exports.getTreeConfigDecoder = getTreeConfigDecoder;
exports.getTreeConfigEncoder = getTreeConfigEncoder;
exports.getTreeConfigSize = getTreeConfigSize;
exports.getUnverifyCollectionInstruction = getUnverifyCollectionInstruction;
exports.getUnverifyCollectionInstructionDataCodec = getUnverifyCollectionInstructionDataCodec;
exports.getUnverifyCollectionInstructionDataDecoder = getUnverifyCollectionInstructionDataDecoder;
exports.getUnverifyCollectionInstructionDataEncoder = getUnverifyCollectionInstructionDataEncoder;
exports.getUnverifyCreatorInstruction = getUnverifyCreatorInstruction;
exports.getUnverifyCreatorInstructionDataCodec = getUnverifyCreatorInstructionDataCodec;
exports.getUnverifyCreatorInstructionDataDecoder = getUnverifyCreatorInstructionDataDecoder;
exports.getUnverifyCreatorInstructionDataEncoder = getUnverifyCreatorInstructionDataEncoder;
exports.getUpdateMetadataInstruction = getUpdateMetadataInstruction;
exports.getUpdateMetadataInstructionDataCodec = getUpdateMetadataInstructionDataCodec;
exports.getUpdateMetadataInstructionDataDecoder = getUpdateMetadataInstructionDataDecoder;
exports.getUpdateMetadataInstructionDataEncoder = getUpdateMetadataInstructionDataEncoder;
exports.getUseMethodCodec = getUseMethodCodec;
exports.getUseMethodDecoder = getUseMethodDecoder;
exports.getUseMethodEncoder = getUseMethodEncoder;
exports.getUsesCodec = getUsesCodec;
exports.getUsesDecoder = getUsesDecoder;
exports.getUsesEncoder = getUsesEncoder;
exports.getVerifyCollectionInstruction = getVerifyCollectionInstruction;
exports.getVerifyCollectionInstructionDataCodec = getVerifyCollectionInstructionDataCodec;
exports.getVerifyCollectionInstructionDataDecoder = getVerifyCollectionInstructionDataDecoder;
exports.getVerifyCollectionInstructionDataEncoder = getVerifyCollectionInstructionDataEncoder;
exports.getVerifyCreatorInstruction = getVerifyCreatorInstruction;
exports.getVerifyCreatorInstructionDataCodec = getVerifyCreatorInstructionDataCodec;
exports.getVerifyCreatorInstructionDataDecoder = getVerifyCreatorInstructionDataDecoder;
exports.getVerifyCreatorInstructionDataEncoder = getVerifyCreatorInstructionDataEncoder;
exports.getVerifyLeafInstruction = getVerifyLeafInstruction;
exports.getVerifyLeafInstructionDataCodec = getVerifyLeafInstructionDataCodec;
exports.getVerifyLeafInstructionDataDecoder = getVerifyLeafInstructionDataDecoder;
exports.getVerifyLeafInstructionDataEncoder = getVerifyLeafInstructionDataEncoder;
exports.getVersionCodec = getVersionCodec;
exports.getVersionDecoder = getVersionDecoder;
exports.getVersionEncoder = getVersionEncoder;
exports.getVoucherCodec = getVoucherCodec;
exports.getVoucherDecoder = getVoucherDecoder;
exports.getVoucherEncoder = getVoucherEncoder;
exports.getVoucherSize = getVoucherSize;
exports.identifyBubblegumAccount = identifyBubblegumAccount;
exports.identifyBubblegumInstruction = identifyBubblegumInstruction;
exports.identifySplAccountCompressionInstruction = identifySplAccountCompressionInstruction;
exports.isAccountCompressionEvent = isAccountCompressionEvent;
exports.isApplicationDataEvent = isApplicationDataEvent;
exports.isChangeLogEvent = isChangeLogEvent;
exports.isConcurrentMerkleTreeHeaderData = isConcurrentMerkleTreeHeaderData;
exports.isLeafSchema = isLeafSchema;
exports.leafSchema = leafSchema;
exports.makeLeaf = makeLeaf;
exports.makeTree = makeTree;
exports.mintCNft = mintCNft;
exports.parseAppendInstruction = parseAppendInstruction;
exports.parseBurnInstruction = parseBurnInstruction;
exports.parseCancelRedeemInstruction = parseCancelRedeemInstruction;
exports.parseCloseEmptyTreeInstruction = parseCloseEmptyTreeInstruction;
exports.parseCompressInstruction = parseCompressInstruction;
exports.parseCreateTreeInstruction = parseCreateTreeInstruction;
exports.parseDecompressV1Instruction = parseDecompressV1Instruction;
exports.parseDelegateInstruction = parseDelegateInstruction;
exports.parseInitEmptyMerkleTreeInstruction = parseInitEmptyMerkleTreeInstruction;
exports.parseInsertOrAppendInstruction = parseInsertOrAppendInstruction;
exports.parseMintToCollectionV1Instruction = parseMintToCollectionV1Instruction;
exports.parseMintV1Instruction = parseMintV1Instruction;
exports.parseRedeemInstruction = parseRedeemInstruction;
exports.parseReplaceLeafInstruction = parseReplaceLeafInstruction;
exports.parseSetAndVerifyCollectionInstruction = parseSetAndVerifyCollectionInstruction;
exports.parseSetDecompressableStateInstruction = parseSetDecompressableStateInstruction;
exports.parseSetDecompressibleStateInstruction = parseSetDecompressibleStateInstruction;
exports.parseSetTreeDelegateInstruction = parseSetTreeDelegateInstruction;
exports.parseTransferAuthorityInstruction = parseTransferAuthorityInstruction;
exports.parseTransferInstruction = parseTransferInstruction;
exports.parseUnverifyCollectionInstruction = parseUnverifyCollectionInstruction;
exports.parseUnverifyCreatorInstruction = parseUnverifyCreatorInstruction;
exports.parseUpdateMetadataInstruction = parseUpdateMetadataInstruction;
exports.parseVerifyCollectionInstruction = parseVerifyCollectionInstruction;
exports.parseVerifyCreatorInstruction = parseVerifyCreatorInstruction;
exports.parseVerifyLeafInstruction = parseVerifyLeafInstruction;
exports.setupSingleVerifiedCNFT = setupSingleVerifiedCNFT;
exports.sparseMerkleTreeFromLeaves = sparseMerkleTreeFromLeaves;
exports.verifyCNft = verifyCNft;
exports.verifyCNftCreator = verifyCNftCreator;
//# sourceMappingURL=index.js.map
//# sourceMappingURL=index.js.map